/*****************************************************************************/
/*                                                                           */
/*  activePacket.h                                                           */
/*                                                                           */
/*                                                                           */
/*     Module:                                                               */
/*             activePacket.so library                                       */
/*                                                                           */
/*     Description:                                                          */
/*             Public header file for the activePacket.c module.             */
/*                                                                           */
/*     History:                                                              */
/*             25/06/2002 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif		/* __cplusplus */

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

#include <errno.h>
#include <string.h>
#ifdef GLOBUS_IO_SUPPORT
#include <globus_io.h>
#endif


/*****************************************************************************/
/* Macros                                                                    */
/*****************************************************************************/

/* Default datagram size used */
#define DEFAULT_DATAGRAM_SIZE 1500

/* Pues eso: display some debug information */
//###FC - NO DEBUG #define activePacket_DEBUG 1

/* Error messages */
#ifdef activePacket_DEBUG
#define activePacket_M_error(num) \
  printf("Error %d at line %d, file %s\n", num, __LINE__, __FILE__)

#define activePacket_M_errno(num) \
  printf("Error %d at line %d, file %s: %s\n", num, __LINE__, __FILE__, strerror(num))
#else
#define activePacket_M_error(num) \
  return(0)
#define activePacket_M_errno(num) \
  return(0)
#endif

/* Types of passive socket (native sockets or Globus_io sockets) */
#define NATIVE_SOCKET 0
#define GLOBUS_SOCKET 1

/* Maximum size of the IP options in the IP header */
#define IP_OPTIONS_SIZE 44


/*****************************************************************************/
/* Types                                                                     */
/*****************************************************************************/

typedef struct activeSocketHandle_s activeSocketHandle_t;
typedef union passiveSocketUnion_u passiveSocketUnion_t;
typedef struct sockConfStruct_s sockConfStruct_t;

//Structure used to save socket configuration before activating it
struct sockConfStruct_s {
  int df;
  int noDelay;
  char ipOptions[IP_OPTIONS_SIZE];
  int ipOptionsLen;
};

//Passive socket handle
union passiveSocketUnion_u {
  int socket;
#ifdef GLOBUS_IO_SUPPORT
  globus_io_handle_t *hGlobusIo;
#else
  char *hGlobusIo;
#endif
};

//Active socket structure
struct activeSocketHandle_s {
  passiveSocketUnion_t socket;
  int socketType;
  int streamType;
  char *anepHeader;
  //uint16_t anepHeaderSize;
  sockConfStruct_t passiveConf;
  size_t leftToRead;
};


/*****************************************************************************/
/* Public functions                                                          */
/*****************************************************************************/

//Open an active socket

extern activeSocketHandle_t *mactiveSocket(void *socket, int type, char *service,char *destIp, int port);
extern activeSocketHandle_t *activeSocket(void *socket, int type, char *service, char *proxyTan, char *destIp);

//Close an active socket
extern int activeClose(activeSocketHandle_t *);

//Write thru an active socket
extern int activeWrite(activeSocketHandle_t *, char *payload, size_t size);

//Read thru an active socket
extern int activeRead(activeSocketHandle_t *, char *payload, size_t *size);


#ifdef __cplusplus
}
#endif		/* __cplusplus */
