/**
 * Demultiplex.java
 *
 * @author Jean-Patrick Gelas
 */

import java.io.*;
import java.net.*;
import java.util.StringTokenizer;
import java.util.Hashtable;

class Demultiplex extends Thread
{
    final static boolean DBG = true;

    TANet net;
    
    // var. UDP
    static Hashtable ports = new Hashtable();



    // TCP [_constructor_]
    Demultiplex( Socket s )
    {
	net = new TANet( s ); 
    }

    // UDP [_constructor_]
    Demultiplex( int port )
    {
	net = new TANet( port );
    }

    public void run()
    {
	Class cl;
	Service ns;
	boolean add_in_list = false;
	
	while(true) {
	    try {
		net.recvANEPkt();
		net.recoverInfoFromANEPkt();
		if(DBG) net.display();
	    } catch (BadANEPversionException e) {
		System.err.println( "Demultiplex (tcp-udp) : " +
				    "Received unsupported ANEP version "+
				    "exception : " + e );
		return;
	    } catch (IOException e) {
		System.err.println( "Demultiplex (tcp-udp) : " + e );
		e.printStackTrace();
		return;
	    } catch (InterruptedException e) {
		System.err.println("Demultiplex : timeout [>"+net.timeout+"]");
		return;
	    }
	    
	    try {
		
		// Test if service 'serviceName' is NOT available
		if( !TAMANOIRd.serviceManager.contains(net.serviceName) ) {
		    TAMANOIRd.serviceManager.download(net.serviceName); // write service on disk
		    add_in_list = true;
		}
		
		String sn = new String(net.serviceName);
		// get root service name
		if(sn.startsWith("http:")) {
		    StringTokenizer stok = new StringTokenizer(sn,"/");
		    stok.nextToken(); // http:	
		    stok.nextToken(); // aaa.bbb.ccc.ddd
		    sn = stok.nextToken(); // only service name
		}
		

		// service is now available
		switch ( net.proto ) {
		case TANet.TCP : // -------------------------- TCP
		    // load service in memory
		    cl = Class.forName( sn );
		    ns = (Service)( cl.newInstance () );
		    if(DBG) System.out.println("Demultiplex : Class.forName("+sn+"), " + "cl.newInstance()");
		    // add service in list of service available
		    if(add_in_list) TAMANOIRd.serviceManager.add(net.serviceName, ns);
		    ns.init( net );
		    ns.start();
		    break;
		case TANet.UDP : // -------------------------- UDP
		    Integer i = new Integer( net.port_tmp );
		    if( !ports.containsKey( i )) {
			cl = Class.forName( sn );
			ns = (Service)( cl.newInstance () );
			ports.put( i , ns );
			if(DBG) System.out.println("Demultiplex : new UDP connection !");
		    }
		    ((Service)(ports.get( i ))).init(net);
		    ((Service)(ports.get( i ))).run();
		    break;
		default :
		    System.err.println("Demultiplex : Unknown protocol !");
		}
		
		
	    } catch (ClassNotFoundException e) { 
		System.err.println( "Demultiplex : " + e ); 
	    } catch (InstantiationException e) { 
		System.err.println( "Demultiplex : " + e ); 
	    } catch (IllegalAccessException e) { 
		System.err.println( "Demultiplex : " + e ); 
	    }

	    if(net.proto == TANet.TCP) break; 
	} /* while (true) */
	if(DBG) System.out.println("Demultiplex : Thread done");
    } // run()
    
} // class Demultiplex



		    //Service ns = (Service)((Service)(TAMANOIRd.serviceManager.get(net.serviceName))).newInstance();
		    //((Service)(TAMANOIRd.serviceManager.get(net.serviceName))).init( net );
		    //((Service)(TAMANOIRd.serviceManager.get(net.serviceName))).start();	
