/**
 *  Route.java
 * 
 *
 *  @author Jean-Patrick Gelas
 */

/* works with "Entry.java"  */

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.*;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class Route
{
    final static boolean DBG = false ; // verbose mode if true

    static InetAddress localhost ;
    static Vector routeTable ; 

    Route( InetAddress localhost, String table_dat ) {
	this.localhost = localhost;
	updateRouteTableFromFile( table_dat );
    }

    static void display() {
	System.out.println( "Route : to -> next"    );
	for(Enumeration e = routeTable.elements(); e.hasMoreElements();) {
	    Entry t = (Entry)e.nextElement();
	    System.out.println ( "Route : " + t.to.toString() + " -> " + t.next.toString());
	}
	System.out.println( "Route : " + routeTable.size() + " entries." );
    } // display()
    

    static InetAddress routeToNode( InetAddress dest ) {
	
	for(Enumeration e = routeTable.elements(); e.hasMoreElements();) {
	    Entry t = (Entry)e.nextElement();
	    if(t.to.equals( dest )) return t.next;
	}
	return null;
    } // routeToNode()

    static InetAddress routeToNode( String str_dest ) {
	try {
	    return routeToNode( InetAddress.getByName(str_dest) );
	} catch (UnknownHostException e) { 
	    System.err.println( "Route : " + e );
	    return null;
	}
    } // routeToNode()


    static void updateRouteTableFromFile( String fileName ) {
	
	routeTable = new Vector();
	if(DBG) System.out.println("Route (updateRouteTableFromFile) fileName = " + fileName );
	try {
	    BufferedReader in = new BufferedReader(new FileReader( fileName ));
	    String str;
	    while((str=in.readLine()) != null ) {
		str = str.trim();
		// ignore comments
		if( !str.startsWith("#") && str.length() != 0 ) { 

		    StringTokenizer st = new StringTokenizer( str );
		    InetAddress from = InetAddress.getByName( st.nextToken() );

		    if(DBG) System.out.println("Route : (from) " + from + 
					       " (to) " + localhost );
		    if(from.equals(localhost)) {
			if(DBG) System.out.println("Route : OK");
			InetAddress to = InetAddress.getByName(st.nextToken());
			InetAddress next = InetAddress.getByName(st.nextToken());
			routeTable.addElement(new Entry( to, next )); 
		    }
		}
	    }
	    in.close();
	} catch (UnknownHostException e) { 
	    System.out.println( "Route : " + e );
	} catch ( IOException e ) {
	    System.out.println( "Route : (updateRouteTableFromFile) " + e );
	    System.exit(1);
	}
    } // updateRouteTableFromFile()
    
} // class Route







