/**
 * Service.java
 *
 * @author Jean-Patrick Gelas
 */

import java.io.*;
import java.net.*;

class Service extends Thread
{
    final static boolean DBG = false;

    TANet net;

    boolean FINISHED;

    Service()
    {
	if(DBG) System.out.println( "Service : build" );
    }

    public void run() 
    {
	if(DBG) System.out.println( "Service : run()" );
    }

    public void init( TANet net ) 
    {
	FINISHED = false;
	this.net = net;

	// cache some data
	net.serviceNameinBytes = net.serviceName.getBytes();
	if(DBG) System.out.println( "Service : init(TANet) " + 
				    net.serviceName );

	// build path for the next hop
	if(net.isANEP) net.buildP2Ppath( net.proxyTANId, net.destId ); 
    }


    public void recv() 
    {
	if(net.proto == TANet.UDP) {
	    FINISHED = true;
	    return;
	}
	try {
	    net.recvANEPkt();
	    net.recoverInfoFromANEPkt();
	} catch (BadANEPversionException e) {
	    System.err.println( "Service : recvANEPkt() (tcp) : Received " +
				"unsupported ANEP version exception : " + e );
	} catch (EOFException e) {
	    if(DBG) System.err.println( "Service : EOF " );
	    FINISHED = true;
	} catch (IOException e) {
	    System.err.println( "Service : recvANEPkt() : " + e );
	    //FINISHED = true;
	} catch (InterruptedException e) {
	    System.err.println("Service (recv) : timeout [>"+net.timeout+"]");
	}
    }

    public void recv_next()  
    {
	//if(net.proto == TANet.UDP) {
	//FINISHED = true;
	//return;
	//}
	try {
	    net.recvANEPkt_next();
	    net.recoverInfoFromANEPkt();
	} catch (BadANEPversionException e) {
	    System.err.println( "Service : recvANEPkt() (tcp) : Received " +
				"unsupported ANEP version exception : " + e );
	} catch (EOFException e) {
	    if(DBG) System.err.println( "Service : EOF " );
	    FINISHED = true;
	} catch (IOException e) {
	    System.err.println( "Service : recvANEPkt() : " + e );
	    //FINISHED = true;
	} catch (InterruptedException e) {
	    System.err.println("Service (recv) : timeout [>"+net.timeout+"]");
	}
    }

    public void forward() throws IOException
    {
  	if(net.isProxyTAN) {
  	    net.sendPayload_next(); // send just payload to the recipient
  	} else {
  	    net.sendANEPkt_next(); // send ANEP pkt to the next TAN
  	}
    }

    public void process() {}

    public ANEPacket getANEPkt()
    {
	return net.anepkt;
    }


} // class Service





