/**
 * TANet.java
 *
 * @author Jean-Patrick Gelas
 */

import java.net.*;
import java.io.*;

class TANet 
{
    final static boolean  DBG = true;

    static final int UDP = 1;
    static final int TCP = 2;
    int proto;
    boolean isANEP = true; 
    
    static InetAddress localhost;
    static String      localhostName;
    int size, count, c;
    
    ANEPacket anepkt;

    static final int SERVICE_NUMBER = 64;

    // variables used for decomposing the ANEP packet
    String srcId;       //  1
    String destId;      //  2
    String lastId;      // 10
    String proxyTANId;  //  6
    String serviceName; // 64
    byte [] serviceNameinBytes;
    



    int payloadSize;
    TLVList TLVs; // use for ANEP packets decomposition

    // TCP variables
    DataOutputStream wr;     // stream links where data come from
    DataInputStream  rd;
    DataOutputStream wr_nxt; // stream links where data goes
    DataInputStream  rd_nxt;

    BufferedReader raw_rd; // text stream (no ANEP packets)
    BufferedWriter raw_wr;
    //DataInputStream raw_rd; // text stream (no ANEP packets)
    //DataOutputStream raw_wr;

    // UDP variables
    DatagramSocket dss, dsr; // s=send, r=recv 
    InetAddress ia_rdwr, ia_rdwr_nxt   ;
    int port_rdwr = -1;
    int port_tmp = -1;
    int port = TAMANOIRd.port;

    boolean isProxyTAN = false; // true if TAN is an edge router
    final static int MAX_SIZE = 1024*64; // max size of the input buffer
    byte [] buffer ;

    int timeout = 0;


    String mesg; // used by ActiveWebS


    // used by : Demultiplex, Service
      // TANet( int proto ) 
    //TANet()   {    }


    // used by : Tcp2udpS
    TANet( int proto, InetAddress nxtTAN, int nxtTANport, int timeout ) 
    {
	init( proto );
	this.timeout = timeout;



	switch(proto) {
   	case TCP: /* TODO but currently unused */
	case UDP:
            try {
                ia_rdwr = nxtTAN;
                dss = new DatagramSocket( /*nxtTANport*/ );
		dss.setSoTimeout( timeout );
            }catch (IOException e) {
                System.err.println( "TANet 1[build udp] : " + e );
            }
	    break;
	} // switch()
	buffer = new byte[MAX_SIZE];
    }



    // used by : Pingtan
    TANet( int proto, String proxyTAN, int proxyTANport, int timeout ) 
    {
	// this( proto );
	init( proto );
	this.timeout = timeout;

	srcId = localhostName;
	lastId = srcId;

	switch(proto) {
   	case TCP:
	    try {
		InetAddress ia_proxyTAN = InetAddress.getByName( proxyTAN );
		Socket s = new Socket( ia_proxyTAN, proxyTANport );
		s.setSoTimeout( timeout ); 
		// option TCP_NODELAY
		s.setTcpNoDelay(true); // p.157 Program. Reseau avec Java
		wr = new DataOutputStream(s.getOutputStream());
		rd = new DataInputStream(s.getInputStream());
	    }catch ( UnknownHostException e ) {
		System.err.println( "TANet 2[build tcp] : " + e );
	    }catch ( IOException e ) {
		System.err.println( "TANet 2[build tcp] : " + e );
	    }
	    break;

	case UDP:
            try {
                ia_rdwr = InetAddress.getByName( proxyTAN );
                dsr = new DatagramSocket( TAMANOIRd.port  );
                dss = new DatagramSocket( );
		System.out.println("TANet (Pingtan) dss.getLocalPort x= " + dss.getLocalPort() );
		dsr.setSoTimeout( timeout );
	    }catch (UnknownHostException e) {
                System.err.println( "TANet 2[build udp] : " + e );
            }catch (IOException e) {
                System.err.println( "TANet 2[build udp] : " + e );
            }
	    break;
	} // switch()
	buffer = new byte[MAX_SIZE];
    } 

    // used by : SendPackets
    TANet( int proto, String proxyTAN, int proxyTANport ) 
    {
	this( proto, proxyTAN, proxyTANport, 0 );
    } 



    // used by : Demultiplex TCP
    TANet( Socket cc )
    {
	// this( TCP );
	init( TCP );
        try {
            // build path with the previous node i.e the node where
            // data come from.
            rd = new DataInputStream(cc.getInputStream());
            wr = new DataOutputStream(cc.getOutputStream());
	    cc.setTcpNoDelay(true);  // 12062002
        } catch (IOException e) {
            System.err.println( "TANet 3: " + e );
        }
	buffer = new byte[MAX_SIZE];
    }

    // used by : Demultiplex_raw TCP (only)
    TANet( int proto, Socket cc, boolean isANEP )
    {
	// this( proto );
	init( proto );
        try {
	    isProxyTAN = true;
	    this.isANEP = isANEP;
	    cc.setTcpNoDelay(true); 
            // build path with the previous node i.e the node where
            // data come from.
	    raw_rd=new BufferedReader(new InputStreamReader(cc.getInputStream()));
	    raw_wr=new BufferedWriter(new OutputStreamWriter(cc.getOutputStream()));
	    //raw_rd=new DataInputStream(cc.getInputStream());
	    //raw_wr=new DataOutputStream(cc.getOutputStream());

        } catch (IOException e) {
            System.err.println( "TANet 4: " + e );
        }
	//buffer = new byte[MAX_SIZE];
    }



    // used by : Demultiplex UDP
    TANet( int port )
    {
	init( UDP );
	this.port = port;
	buffer = new byte[MAX_SIZE];
	try {
	    dsr = new DatagramSocket( port );
	    dss = new DatagramSocket( );
	}catch (IOException e) {
	    System.err.println( "TANet [build 2 udp] : " + e );
	}
    }    

    // used by : Tcp2udpS
    TANet()
    {
	// this( UDP );
	init( UDP );
	this.port = port;
	buffer = new byte[MAX_SIZE];
	try {
	    dss = new DatagramSocket();
	}catch (IOException e) {
	    System.err.println( "TANet [build 3 udp] : " + e );
	}
    }    


    private void init( int proto ) 
    {
	if(proto == UDP || proto == TCP) {
	    this.proto = proto;
	    try {
		// init localhost[Name] 
		if(TAMANOIRd.frontendName == null) {
		    localhost = InetAddress.getLocalHost();
		} else {
		    localhost = InetAddress.getByName(TAMANOIRd.frontendName);
		}
		localhostName = localhost.getHostAddress();

		if(DBG) System.err.println("TANet : localhost is " + 
					   localhostName);
	    } catch (UnknownHostException e) {
		System.err.println( "TANet 5: " + e );
	    }
	} else {
	    System.err.println("TANet (init) : unknown protocol");
	    System.exit(0);
	}
    }















    // used by : RecvPackets
    public static int init_session( String proxyTANname, 
				    int    proxyTANport,
				    String serverName,
				    int    serverPort
				    )
    {
	// client send a message to the stream server, wich mean
	// something like that : Hello I'm the client,
	// 'localhostname', my proxy TAN is, 'proxyTANname' and I
	// would like you send me a stream.
        try {
	     if(TAMANOIRd.frontendName == null) {
		 localhostName = InetAddress.getLocalHost().getHostAddress();
	     } else {
		 localhostName = InetAddress.getByName(TAMANOIRd.frontendName).getHostAddress();
	     }

            InetAddress ia_server = InetAddress.getByName( serverName );
	    System.out.println("init_session::"+InetAddress.getByName( serverName ));

            Socket s = new Socket( ia_server, serverPort );
	    System.out.println("socket::"+s.getInetAddress().getHostAddress());
	    s.setTcpNoDelay(true); // 12062002

            DataOutputStream wr = new DataOutputStream(s.getOutputStream());
            wr.writeUTF( localhostName ); // I'm ...localhostName (client)
            wr.writeUTF( proxyTANname );  // my proxyTAN is ...proxyTANname,
	    wr.writeInt( proxyTANport );  // running on port ..proxyTANport.
	    wr.close();

	    if(DBG) System.err.println("TANet (init_session): host is " + 
				       localhostName);
        }catch (UnknownHostException e) {
            System.err.println( "TANet (init_session) : " + e );
        }catch (IOException e) {
            System.err.println( "TANet (init_session) : " + e );
        }
	return 0;
    }
    

    // Build path for the next hop
    public int buildP2Ppath(String proxyTANId, String destId)
    {
	try {
	    // These first lines are useful when destId is a TAN (Used by
	    // the Pingtan application).
	    InetAddress ia_destId = InetAddress.getByName( destId ); 
	    if(ia_destId.equals(localhost)) {
		if(DBG) System.out.println("TANet (buildP2P) : 0");
		return 0;
	    }

	    InetAddress ia_nxt = 
		TAMANOIRd.route.routeToNode(ia_destId);
	    if(ia_nxt != null) { 
		// Packet is intend for a TAN
		switch(proto) {
		case TCP:
		    Socket s = new Socket( ia_nxt, port );
		    wr_nxt = new DataOutputStream(s.getOutputStream());
		    rd_nxt = new DataInputStream(s.getInputStream());
		    s.setTcpNoDelay(true); // 12062002
		    break;
		case UDP:
		    ia_rdwr_nxt = ia_nxt;
		    break;
		}
	    } else { // (ia_nxt == null)
		// destId is NOT a TAN
		InetAddress ia_proxyTANId = InetAddress.getByName(proxyTANId);
		if(ia_proxyTANId.equals( localhost )) {
		    isProxyTAN = true;
		    if(DBG) System.out.println("TANet buildP2P : "+ 
					       localhostName +
					       " is a proxyTAN.");
		    switch(proto) {
		    case TCP:
			Socket s = new Socket( ia_destId, port );
			wr_nxt = new DataOutputStream(s.getOutputStream());
			rd_nxt = new DataInputStream(s.getInputStream());
		    break;
		    case UDP:
			ia_rdwr_nxt = ia_destId;
			break;
		    }
		} else {
		    // next hope is a TAN
		    isProxyTAN = false;
		    if(DBG) System.out.println("TANet buildP2P : "+ 
					       localhostName +
					       " is NOT a proxyTAN.");
		    InetAddress ia_nxt_tan = 
			TAMANOIRd.route.routeToNode(ia_proxyTANId);
		    if(DBG) System.out.println( "TANet:ia_nxt_tan= "+
						ia_nxt_tan); 
		    switch(proto) {
		    case TCP:
			Socket s = new Socket( ia_nxt_tan, port );
			wr_nxt = new DataOutputStream(s.getOutputStream());
			rd_nxt = new DataInputStream(s.getInputStream());
			s.setTcpNoDelay(true); // 12062002
			break;
		    case UDP:
			ia_rdwr_nxt = ia_nxt_tan ;
			break;
		    }
		}
	    }
	}catch (UnknownHostException e) {
	    System.err.println( "TANet  : " + e );
	}catch (IOException e) {
	    System.err.println( "TANet  : " + e );
	}
	return 0;
    }




    public ANEPacket recvANEPkt() throws 
	IOException, 
	BadANEPversionException, 
	InterruptedException
    {
	switch(proto) {
	case TCP:
	    size = rd.readInt(); // read size of data incoming
	    count = 0;
	    while (count < size ) {
		c = rd.read( buffer, count, size - count );
		if(c<0) break; // client socket has been closed
		count += c;
		// if(DBG) System.out.println( "TANet recv() (tcp): "+
		// "trx = ["+count+"] + " + c );
	    }
	    // if(DBG) System.out.println( "TANet recv() (tcp): rcx "+
	    // "completed ["+ count +" Bytes]" );
	    anepkt = ANEPacket.unmarshall( buffer );
	    // if(DBG) System.out.println( "TANet recv() : " + 
	    // anepkt.payload.length + " Bytes");
	    
	    TAMANOIRd.recv_tcp_counter++; // for statistics
	    TAMANOIRd.inputsize_tcp_counter += count;
	    break;

	case UDP:
	    DatagramPacket dp = new DatagramPacket (buffer, MAX_SIZE);
	    dsr.receive(dp); 
	    ia_rdwr = dp.getAddress(); // get address where data come from

	    // l'emetteur a utilise port_tmp pour emettre ses donnees
	    port_tmp = dp.getPort();
	    System.out.println("TANet : RecvANEPkt port_tmp = "+port_tmp);
	    
	    anepkt = ANEPacket.unmarshall(dp.getData());
	    

	    TAMANOIRd.recv_udp_counter++; // for statistics
	    TAMANOIRd.inputsize_udp_counter += dp.getLength();
	    break;
	}
	return anepkt;
    } // recvANEPkt()

    public ANEPacket recvANEPkt_next() throws 
      IOException, 
      BadANEPversionException, 
      InterruptedException
    {
	switch(proto) {
	case TCP:
	    size = rd_nxt.readInt(); // read size of data incoming
	    count = 0;
	    while (count < size ) {
		c = rd_nxt.read( buffer, count, size - count );
		if(c<0) break; // client socket has been closed
		count += c;
		// if(DBG) System.out.println( "TANet recv() (tcp): "+
		// "trx = ["+count+"] + " + c );
	    }
	    // if(DBG) System.out.println( "TANet recv() (tcp): rcx "+
	    // "completed ["+ count +" Bytes]" );
	    anepkt = ANEPacket.unmarshall( buffer );
	    // if(DBG) System.out.println( "TANet recv() : " + 
	    // anepkt.payload.length + " Bytes");
	    TAMANOIRd.recv_tcp_counter++; // for statistics
	    TAMANOIRd.inputsize_tcp_counter += count;
	    break;

	case UDP:
	    DatagramPacket dp = new DatagramPacket (buffer, MAX_SIZE);
	    dsr.receive(dp);
	    ia_rdwr = dp.getAddress(); // get address where data come from
	    anepkt = ANEPacket.unmarshall(dp.getData());

	    TAMANOIRd.recv_udp_counter++; // for statistics
	    TAMANOIRd.inputsize_udp_counter += dp.getLength();
	    break;
	}
	return anepkt;
    } // recvANEPkt_next()

    // send only the payload to the destId (recipient)
    public int sendPayload_next() throws IOException
    {
	switch(proto) {
	case TCP :
	    wr_nxt.writeInt(anepkt.payload.length);
	    wr_nxt.write(anepkt.payload, 0, anepkt.payload.length);
	    // wr_nxt.flush();
	    TAMANOIRd.send_tcp_counter++; // for statistics
	    TAMANOIRd.outputsize_tcp_counter += anepkt.payload.length;
	    break;
	case UDP :
	    //System.out.println("TANet sendPayload_next()
	    //ia_rdwr_nxt=" + ia_rdwr_nxt);
	    dss.send(new DatagramPacket (anepkt.payload, anepkt.payload.length,
                                        ia_rdwr_nxt, port ));

	    TAMANOIRd.send_udp_counter++; // for statistics
	    TAMANOIRd.outputsize_udp_counter += anepkt.payload.length;
	    break;
	}
	if(DBG) System.out.println( "TANet sendPayload_next() : " +
				    anepkt.payload.length +" Bytes" );
	return 0;
    } // sendPayload_next()


    // use by Tcp2udpS
    public int sendPayload() throws IOException
    {
	switch(proto) {
	case TCP :
	    wr.writeInt(anepkt.payload.length);
	    wr.write(anepkt.payload, 0, anepkt.payload.length);
	    // wr_nxt.flush();
	    TAMANOIRd.send_tcp_counter++; // for statistics
	    TAMANOIRd.outputsize_tcp_counter += anepkt.payload.length;
	    break;
	case UDP :
	    //System.out.println("TANet sendPayload_next()
	    //ia_rdwr_nxt=" + ia_rdwr_nxt);
	    dss.send(new DatagramPacket (anepkt.payload, anepkt.payload.length,
                                        ia_rdwr, port ));

	    TAMANOIRd.send_udp_counter++; // for statistics
	    TAMANOIRd.outputsize_udp_counter += anepkt.payload.length;
	    break;
	}
	if(DBG) System.out.println( "TANet sendPayload() : " +
				    anepkt.payload.length +" Bytes" );
	return 0;
    } // sendPayload()


    // receive only payload (not ANEP packet)
    public byte [] recvPayload_next() throws IOException, InterruptedException
    {
	switch(proto) {
	    case TCP:
	    size = rd_nxt.readInt(); // read size of data incoming
	    count = 0;
	    while (count < size ) {
		c = rd_nxt.read( buffer, count, size - count );
		if(c<0) break; // client socket has been closed
		count += c;
		// if(DBG) System.out.println( "TANet recv() (tcp): "+
		// "trx = ["+count+"] + " + c );
	    }
	    // if(DBG) System.out.println( "TANet recv() (tcp): rcx "+
	    // "completed ["+ count +" Bytes]" );
	    // if(DBG) System.out.println( "TANet recv() : " + 
	    // anepkt.payload.length + " Bytes");
	    //TAMANOIRd.recv_tcp_counter++; // for statistics
	    //TAMANOIRd.inputsize_tcp_counter += count;
	    return buffer;
	    
	    case UDP:
	    DatagramPacket dp = new DatagramPacket (buffer, MAX_SIZE);
	    dsr.receive(dp);
	    ia_rdwr = dp.getAddress(); // get address where data come from
	    TAMANOIRd.recv_udp_counter++; // for statistics
	    TAMANOIRd.inputsize_udp_counter += dp.getLength();
	    return (dp.getData());
	}
	return null;
    } // recvANEPkt_next()



    // send an ANEP packet to the node where data come from. 
    public int sendANEPkt() throws IOException
    {
	byte [] buffer = anepkt.marshall();
	switch(proto) {
	case TCP:
	    wr.writeInt(buffer.length);
	    wr.write(buffer, 0, buffer.length); 
	    TAMANOIRd.send_tcp_counter++; // for statistics
	    TAMANOIRd.outputsize_tcp_counter += buffer.length;
	    break;
	case UDP:
	    System.out.println("TANet : ia_rdwr = " + ia_rdwr);
	    System.out.println("TANet : dss = " + dss);
	    dss.send(new DatagramPacket (buffer, buffer.length, ia_rdwr, port));
	    System.out.println("TANet sent data on port = " + port +
			       " from port = " +dss.getLocalPort() );

	    TAMANOIRd.send_udp_counter++; // for statistics
	    TAMANOIRd.outputsize_udp_counter += buffer.length;
	    break;
	}
	return 0;
    } // sendANEPkt()
    

    // send an ANEP packet to the next TAN
    public int sendANEPkt_next() throws IOException
    {
	    byte [] buffer = anepkt.marshall();
	    switch(proto) {
	    case TCP:
		wr_nxt.writeInt(buffer.length);
		wr_nxt.write(buffer, 0, buffer.length);
		TAMANOIRd.send_tcp_counter++; // for statistics
		TAMANOIRd.outputsize_tcp_counter += buffer.length;
		break;
	    case UDP:
		dss.send(new DatagramPacket (buffer, buffer.length,
					    ia_rdwr_nxt, port));
		TAMANOIRd.send_udp_counter++; // for statistics
		TAMANOIRd.outputsize_udp_counter += buffer.length;
		break;
	    } // switch()
	    if(DBG) System.out.println( "TANet sendANEPkt_next() : " +
					buffer.length +" Bytes" );
	    return 0;
    }


    // build an ANEP packet
    public int makeANEPkt( byte [] srvNameinBytes,
			   byte [] opt1,  /* srcId in Bytes      */
			   byte [] opt2,  /* destId in Bytes     */
			   byte [] opt10, /* lastId in Bytes     */
			   byte [] opt6,  /* proxyTANId in Bytes */
			   byte [] payload 
			   )
    {
	TLVList TLVs = TLVList.Cons( 
          new TLV( SERVICE_NUMBER, srvNameinBytes.length, srvNameinBytes ), 
	    TLVList.Cons( new TLV( 1, opt1.length, opt1 ),  /* Source Id */
              TLVList.Cons( new TLV( 2 , opt2.length, opt2), /* Dest Id */
               TLVList.Cons( new TLV( 6 , opt6.length, opt6), /* proxyTAN Id */
                TLVList.Cons( new TLV( 10 , opt10.length, opt10), /* Last Id */
                 TLVList.Nil )))));
	  anepkt = new ANEPacket( ANEPacket.TAMANOIRtypeId, TLVs, payload );
	  return 0;
    }

    public byte[] getBuffer() 
    {
	return buffer;
    }
    public ANEPacket getANEPkt() 
    {
	return anepkt;
    }

    
    public static InetAddress getlocalInetAddress()
    {
        try {
	    if(TAMANOIRd.frontendName == null) {
		return InetAddress.getLocalHost();
	    } else {
		return InetAddress.getByName(TAMANOIRd.frontendName);
	    }
        } catch (UnknownHostException e) {
            System.err.println( "TANet.getlocalInetAddress: " + e );
	    return null;
        }
    }
    
    public void close()
    {
	switch(proto) {
	case TCP:
	    try {
		if(wr != null) wr.close();
		if(rd != null) rd.close();
		if(wr_nxt != null) wr_nxt.close();
		if(rd_nxt != null) rd_nxt.close();
		if(raw_rd != null) raw_rd.close();
		if(raw_wr != null) raw_wr.close();
	    } catch (IOException e) {
		System.err.println( "TANet : close() :" + e );
	    }
	    break;

	case UDP:
	    //!\\ Risque d'empecher PINGTAN de fonctionner
	    //if(dss != null) dss.close();
	    break;
	}
    }
    
    /* ********************************************** */

    // display info about a packet ANEP (for debugging purpose)
    public void display() 
    {
	System.out.println("*-------------------------------"+
			   "\r\n* ANEP info   : "+ serviceName+ 
			   "\r\n* srcId       : "+ srcId+
			   "\r\n* destId      : "+ destId+
			   "\r\n* lastId      : "+ lastId+
			   "\r\n* proxyTANId  : "+ proxyTANId+
			   "\r\n* payload size: "+ payloadSize+
			   "\r\n*-------------------------------"
			   );
    }

    public int recoverInfoFromANEPkt() 
    {
	if(anepkt == null) return 0;
	payloadSize = anepkt.payload.length;
	TLVs = anepkt.getTLVs ();
        if (TLVs.length() != 0) {
	    boolean b = true;
	    while ( b ) { // while the TLVlist is not finished
		// if(DBG) System.out.println ("type=" + TLVs.head().getType() + " len=" + TLVs.head().getLength());
		switch ( TLVs.head().getType() ) {
		    /* ***************************************************** */
		    /* RESERVED VALUES cf. RFC (draft) Active Networks Group */
		    /* ***************************************************** */
                case 1 : // # 1 # Source identifier
		    srcId = value2Strings( TLVs.head() );
		    break;
		    
                case 2 : // # 2 # Destination identifier
		    destId = value2Strings( TLVs.head() );
		    break;

                case 3 : // # 3 # Integrity Checksum
		    /* TODO */
		    break;

                case 4 : // # 4 # N/N Authentication
		    /* TODO */
		    break;
		    /* ***************************************************** */
		    /* End of RESERVED VALUES                                */
		    /* ***************************************************** */

		case 6 : // # 6 # proxy TAN Identifier
		    proxyTANId = value2Strings( TLVs.head() );
		    break;

                case 10 : // # 10 #  Last Identifier (last TAN crossed)
		    lastId = value2Strings( TLVs.head() );
		    break;

		case TANet.SERVICE_NUMBER :
		    serviceName = value2Strings( TLVs.head() );
		    break;

		default:
		    System.err.println("Demultiplex (recoverInfoFromANEPkt): unknown Type of TLV.");
		} // switch()
		if (TLVs.tail() != TLVList.Nil) 
		    TLVs = TLVs.tail (); // next TLV
                else 
		    b = false; // list of TLVs finished
	    } // while(b)
	} // if()
	return 0;
    } // recoverInfoFromANEPkt()    

    public static String value2Strings( TLV t ) 
    {
	String tmp = new String(t.getValue());
	return tmp.substring(0,t.getLength ());
    }

} // class TANet


