package ibp;

//
// CapAttribute.java
//
// Written by Yong Zheng.  May 2001
//

/**
  * This class holds the attributes of the capability.
  *
  * @author Yong Zheng
  */


import java.util.*;


public class CapAttribute {
	// constructor
	public CapAttribute ( long dur, int reli, int type ) throws IBPException {
		setDuration(dur);
		setReliability(reli);
		setType(type);	
	}
	
	public CapAttribute ( Date date, int reli, int type) throws IBPException {
		setDuration(date);
		setReliability(reli);
		setType(type);
	};
	
	// fields
	private	long	duration;
	private int	reliability;
	private int	type;
	  	
	// methods
	public void setDuration( Date date ) throws IBPException {
		
		if ( date == null ){
			duration = 0;
			return;
		};
		
		Date curTime = new Date();
		
		if ( !curTime.before(date) ){
			throw new IBPIllegalArgumentException();
		};
		
		duration = date.getTime();
	};
	
	public void setDuration( long dur ) throws IBPException {
		Date curTime = new Date();
		if ( curTime.getTime() >= dur*1000 && dur > 0 ) {
			throw new IBPIllegalArgumentException();
		};
		
		duration = dur*1000;
	};
	
	public void setReliability( int reli ) throws IBPException{
		if ( reli != IBPProtocol.IBP_STABLE && 
	                  reli != IBPProtocol.IBP_VOLATILE ){
			throw new IBPIllegalArgumentException();	  
		};
		reliability = reli;
	};
	
	public void setType( int type) throws IBPException{
		if ( type != IBPProtocol.IBP_BYTEARRAY && 
		     type != IBPProtocol.IBP_CIRQ &&
		     type != IBPProtocol.IBP_FIFO && 
		     type != IBPProtocol.IBP_BUFFER ){
		  	throw new IBPIllegalArgumentException();   
		};
		
		this.type = type;     
	};
	
	public long getDuration( ) {
		return duration/1000;
	};
	
	public int getReliability (){
		return reliability;
	};
	
	public int getType(){
		return type;
	};
	
	public String toString(){
		return("Duration = " + getDuration() + "\n" +
		       "Reliability = " + getReliability() + "\n" +
		       "Type = " + getType() ); 
	};
	
	long	getLifeTime() throws IBPException{
		long tmp;
		
		if ( duration == 0 )
			return 0;
		tmp = duration - new Date().getTime();
		if ( tmp < 0 ){
			throw new IBPIllegalArgumentException();
		}; 
		return(tmp);
	};
		
	public static void main( String[] args ){
		
	};
}; 
