package ibp;

import java.util.*;
import java.net.*;

//
// Capability.java
//
// Written by Yong Zheng

/**
  * This class presents a subclass for ReadCap, WriteCap
  * and ManageCap.
  * 
  * @author   Yong Zheng
  */

class Capability {
	// constructor
	
	/**
	  * creates a capability object
	  *
	  *	@param  cap 	cap in string
	  */
	Capability ( String cap) throws IBPException,UnknownHostException {
		StringTokenizer parser = 
			new StringTokenizer(cap,":/");
		try {
			// get "ibp"
			String str = parser.nextToken();
			if ( str.compareToIgnoreCase("IBP") != 0 )
				throw new IBPMalFormatCapException();
			// get hostname
			String hn = parser.nextToken();
			str = parser.nextToken();
			// get port number
			int port = Integer.parseInt(str.trim());
			depot = new IBPDepot( hn,port);
			// get capability name
			name = parser.nextToken();
			// get key
			key = parser.nextToken();
			// get type
			type = parser.nextToken().trim().toUpperCase();
		}catch (NoSuchElementException e){
			throw new IBPMalFormatCapException();
		}catch (NumberFormatException e){
			throw new IBPMalFormatCapException();
		};
	};
	
	// class fields
	// IBP depot
	private IBPDepot depot;
	// capability name
	private String 	name;
	// capability key
	private String  key;
	// capability type
	private String  type;
	// client timeout
	private int cliTmOut = 0;
	// server timeout
	private int svrTmOut = 0;
	
	
	//methods
	
	/**
	  * return the cap in string
	  */
	public String toString (){
		return("IBP://"+getDepot().getHostName() +":"+getDepot().getPort()+"/"+name+"/"+key+"/"+type+"/");
	};
	
	/**
	  * setCapTimeout set timeout value 
	  *
	  * @param cliTm	cient time out
	  * @param svrTm	server time out
	  * @return No return
	  * @Exception throws IBPIllegalArgumentException
	  */
	public void setCapTimeout( int cliTm, int svrTm) 
				throws IBPIllegalArgumentException{
		if ( cliTm < 0 || svrTm < 0)
			throw new IBPIllegalArgumentException();
		cliTmOut = cliTm;
		svrTmOut = svrTm;	
	};
	
	/**
	  * getClientTimeout get the client side timeout value
	  *
	  * @param no parameter
	  * @return no return
	  * @exception no exception
	  */
	public int getClientTimeout( ) { return cliTmOut; };
		  
	/**
	  * getServerTimeout get the client side timeout value
	  *
	  * @param no parameter
	  * @return no return
	  * @exception no exception
	  */
	public int getServerTimeout( ) { return svrTmOut; };
		  
	
	protected String getName(){ return name;};  
	
	protected String getKey() { return key;};
	
	protected String getType() { return type; };
	
	protected IBPDepot getDepot() { return depot;};
};  
