package ibp;
//
// DepotStatus.java
//
// Written by Yong Zheng.  May 2001
//

/** 
   * This class holds the status of the IBP depot.
   *
   *  @author	Yong Zheng
   */
import java.util.*;

public class DepotStatus {
	//constructor
	DepotStatus ( long stbStor, long stbStorUsed, long volStor, 
	              long volStorUsed, long dur ) throws IBPException {
		setStbStorSize( stbStor);
		setStbStorUsedSize( stbStorUsed);
		setVolStorSize( volStor);
		setVolStorUsedSize( volStorUsed);
		setDuration(dur);	      
	};
	
	DepotStatus ( long stbStor, long volStor, long dur) throws IBPException{
		setStbStorSize(stbStor);
		setVolStorSize(volStor);
		setDuration(dur);
	};
	
			      	
	//class fields
	private long	stbStorSize;
	private long	stbStorUsedSize;
	private long	volStorSize;
	private long	volStorUsedSize;
	private long	duration;
	
	//methods
	public void setStbStorSize( long size ) throws IBPException {
		if (( size < 0 ) || ( size < stbStorUsedSize )) 
			throw new IBPIllegalArgumentException();
		stbStorSize = size;	
	};
	
	public void setVolStorSize ( long size ) throws IBPException {
		if (( size < 0 ) || ( size < volStorUsedSize ))
			throw new IBPIllegalArgumentException();
		volStorSize = size;	
	};
	
	void setStbStorUsedSize ( long size ) throws IBPException {
		if (( size < 0) || ( size > stbStorSize ))
			throw new IBPIllegalArgumentException();
		stbStorUsedSize = size;	
	};
	
	void setVolStorUsedSize( long size) throws IBPException {
		if (( size < 0 ) || ( size > volStorSize))
			throw new IBPIllegalArgumentException();
		volStorUsedSize = size;	
	};
	
	public void setDuration ( long dur ) throws IBPException {
		duration = dur;	
	};
	
	public long getStbStorSize() { return stbStorSize; };
	public long getStbStorUsedSize() { return stbStorUsedSize; };
	public long getVolStorSize() { return volStorSize; };
	public long getVolStorUsedSize() { return volStorUsedSize; };
	public long getDuration() { return duration ;};
	
	public String toString(){
		return("Stable Storage size = " + getStbStorSize() +
		"\nUsed Stable Storage size = " + getStbStorUsedSize() +
		"\nVolatile Storage size = " + getVolStorSize() + 
		"\nUsed Volatile Storage size = " + getVolStorUsedSize() +
		"\nDuration = " + getDuration() + "\n");
	};
	
	public static void main ( String[] args){
		try {
			DepotStatus dpt1 = new DepotStatus(100,10,200,150, 300);
			
			System.out.println(""+dpt1);
		} catch ( IBPException e) {
			System.err.println("IBP Error!");
			System.exit(-1);
		};		
	};
};
