package ibp;

import java.io.*;
import java.util.*;
import java.net.*;

//
// IBPCaps.java
//
// Written by Yong Zheng
//

/**
  * This class presents a set of capabilites 
  * 
  * @author	Yong Zheng
  */
  
public class IBPCaps {
	//constructor
	
	/**
	  * Creates a set of IBP capabilities with specified size and 
	  * attributes
	  *
	  *	@param	dpt	IBP Depot
	  *	@param	size	Wanted Storage size
	  *	@param  attr	Cap attribute
	  *	@param	cliTm	client side timeout
	  * @param	svrTm	server side timeout
	  */
	public IBPCaps(IBPDepot dpt, long size, CapAttribute attr, 
			int cliTm,  int svrTm) throws IOException,IBPException,UnknownHostException{
		depot = dpt;
		// create a ibp session 
		IBPSession session = new IBPSession(depot.getAddress(),depot.getPort(),cliTm);
		// make request command
		createCaps(session,size,attr,svrTm);	
	};

	/**
	  * Creates a set of IBP capabilities with specified size and 
	  * attributes
	  *
	  *	@param	host	host name
	  *	@param  port    port number
	  *	@param	size	Wanted Storage size
	  *	@param  attr	Cap attribute
	  *	@param	cliTm	client side timeout
	  * @param	svrTm	server side timeout
	  */
	public IBPCaps(String host, int port, long size, CapAttribute attr, 
			int cliTm,  int svrTm) throws IOException,IBPException,UnknownHostException{
		depot = new IBPDepot(host,port);
		// create a ibp session 
		IBPSession session = new IBPSession(depot.getAddress(),depot.getPort(),cliTm);
		// make request command
		createCaps(session,size,attr,svrTm);	
	};		
	
	//class fields
	
	// read capability
	private ReadCap readCap;
	// write capability 
	private WriteCap writeCap;
	// manage capability
	private ManageCap manageCap;
	// IBP Depot
	private IBPDepot depot;
	
	// methods
	
	/**
	  * return a readCap object
	  */
	public ReadCap getReadCap(){
		return readCap;
	};
	
	/**
	  * return a writeCap object
	  */
	public WriteCap getWriteCap(){
		return writeCap;
	};
	
	/**
	  * return a manageCap object
	  */
	public ManageCap getManageCap(){
		return manageCap;
	};
	
	/**
	  * return a readCap in string
	  */
	
	public String exportReadCap(){
		return readCap.toString();
	};     
	
	/**
	  * return a writeCap in string
	  */
	public String exportWriteCap(){
		return writeCap.toString();
	};
	
	/**
	  * return a manageCap in string
	  */
	public String exportManageCap(){
		return manageCap.toString();

	};
	
	/**
	  * return a IBP depot object
	  */
	public IBPDepot getDepot() { return depot;};
	  
	
	public String toString(){
		return "IBP Capability:\n"+
		       "Read  : "+readCap+"\n"+
		       "Write : "+writeCap+"\n"+
		       "Manage: "+manageCap;
	};
	
	public static void main( String[] args){
		String str1 = "This is a simple test, OK!";
		int port1 = 0, port2 = 0;
		if (args.length < 4) {
			System.err.println("Usage:\n\tIBPCaps <hostname1 port1 hostname2 port2>");
			System.exit(-1);
		};
		try {
			port1 = Integer.parseInt(args[1]);
			port2 = Integer.parseInt(args[3]);
		}catch (NumberFormatException e){
			System.err.println("Malformated port number!");
			System.exit(-1);
		};
		
		try {
			IBPDepot dpt = new IBPDepot( args[0],port1);
			IBPDepot dpt1 = new IBPDepot ( args[2],port2);
			CapAttribute attr = new CapAttribute(0,IBPProtocol.IBP_STABLE,
						IBPProtocol.IBP_BYTEARRAY);
			IBPCaps caps = new IBPCaps(dpt,1000,attr,0,0);
			IBPCaps caps1 = new IBPCaps(dpt1,1000,attr,0,0);
			//IBPCaps cap1 = new IBPCaps("hydra4a.cs.utk.edu",11211,1000,attr,0,0);
			System.out.println(""+caps+"\n"+caps1);			
			WriteCap wc = caps.getWriteCap();
			ReadCap rc = caps.getReadCap();
			ManageCap mc = caps.getManageCap();
			WriteCap wc1 = caps1.getWriteCap();
			ReadCap rc1 = caps1.getReadCap();
			ManageCap mc1 = caps.getManageCap();
			
			// test IBP_store functionality
			byte[] ar1 = str1.getBytes();
			System.out.println("Send Data:"+str1);
			long ret = wc.write(ar1,ar1.length);
			/*
			ret = wc.write(ar1,ar1.length);
			ret = wc.write(ar1,ar1.length);
			ret = wc.write(ar1,ar1.length);
			ret = wc.write(ar1,ar1.length);
			*/
			
			//System.exit(-1);
			
			// test IBP_load functionality
			byte[] ar2 = new byte[10000];
			long ret1 = rc.read(ar2,ret*2,0);
			//System.out.println("receieve len:"+ret1);
			//for (int i=0;i<ret1;i++)
			//	System.out.print(""+ar2[i]);
			System.out.println("");	
			System.out.println("Received Data:"+new String(ar2));
			
			// test IBP_copy functionality
			byte[] ar3 = new byte[10000];
			rc.copyTo(wc1,ret*2,0,0,0);
			System.out.println("copy len:"+rc1.read(ar3,ret,0));
			System.out.println("Received data from remote depot:"+new String(ar3));
			wc1.write(ar3,(int)ret);
			
			//System.exit(-1);
			// test IBP_manage functionality
			CapStatus capSt = mc.getCapStatus();
			System.out.println("Before INC\n"+capSt);		   
			mc.incRefer(IBPProtocol.IBP_READCAP);
			mc.incRefer(IBPProtocol.IBP_WRITECAP);
			capSt = mc.getCapStatus();		   
			System.out.println("After INC\n"+capSt);
			mc.decRefer(IBPProtocol.IBP_READCAP);
			capSt = mc.getCapStatus();		   
			System.out.println("After DEC\n"+capSt);
			capSt.setMaxSize(capSt.getMaxSize()+1000);
			capSt.getCapAttr().setDuration(new Date().getTime() + 100000);
			mc.changeCapStatus(capSt);
			System.out.println("Before Change:\n"+capSt);
			
			// test IBP_status functionality
			DepotStatus st = dpt.getStatus("IBP",0,0);
			System.out.println(""+st);
			DepotStatus st1 = dpt.setLimit("ibp",st.getStbStorSize()+500,st.getVolStorSize()+500,-1,0,0);
			System.out.println("After Change:\n");
			System.out.println(""+st1);
			st1.setVolStorSize(st1.getVolStorSize()+100);
			st1.setStbStorSize(st1.getStbStorSize()+100);
			st1 = dpt.setLimit(st1);
			System.out.println(""+st1);
		
		}catch ( IBPException e){
			System.err.println("IBP Error!");
			System.exit(-1);
		}catch ( IOException e){
			System.err.println("IO Error!");
			System.exit(-1);
		}catch ( Exception e){
			System.err.println("Unknown Error!");
			e.printStackTrace(System.err);
			System.exit(-1);
		};
		
		System.out.println("SUCCESS!");		
	};
	
	private void createCaps( IBPSession session, long size, CapAttribute attr, int svrTm) 
									throws IBPException, IOException{
		String cmd = ""+IBPProtocol.IBP_v031+" "+IBPProtocol.IBP_ALLOCATE+" "+
			     attr.getReliability()+" "+attr.getType()+" "+attr.getLifeTime()+" "+
			     size+" "+svrTm+"\n";
		session.putCommUnit(new Request(cmd));
		session.putCommUnit(new Response1());
		// do transaction
		session.perform();
		// get result
		System.out.println("We get:"+session.getResult());
		StringTokenizer parser = new StringTokenizer(session.getResult(),"\r\n ");
		try {
			// get return value and check error code
			String str = parser.nextToken();
			int     retVal;
			retVal = Integer.parseInt(str.trim());
			if ( retVal != IBPProtocol.IBP_OK )
				throw IBPUtil.getException(retVal);
			// get read capability
			str = parser.nextToken();
			System.out.println("we get:"+str);
			readCap = new ReadCap(str);
			// get write capability
			str = parser.nextToken();
			System.out.println("we get:"+str);
			writeCap = new WriteCap(str);
			// get manage capability
			str = parser.nextToken();
			System.out.println("we get:"+str);
			manageCap = new  ManageCap(str);	
		
		}catch (NoSuchElementException e){
			throw new IBPResponseFormatException();
		}catch (NumberFormatException e){
			throw new IBPResponseFormatException();
		};			     
	};				
};	    
	  
