package ibp;

//
// IBPDepot.java
//
// Written by Yong Zheng.  May 2001
//

/** 
   * IBPDepot represents a IBP Server
   *
   *  @author	Yong Zheng
   */
import java.util.*;
import java.net.*;
import java.io.*;

public class IBPDepot {
	// constructor
	public IBPDepot( InetAddress addr, int port){
		address = addr;
		this.port = port;
		try {
			hostname = addr.getHostName();
		}catch (SecurityException e){
			hostname = addr.toString();
		};	 
	};
	public IBPDepot( String hostname, int port) throws UnknownHostException {
		address = InetAddress.getByName(hostname);
		this.port = port;
		this.hostname = hostname;
	};
	public IBPDepot( InetAddress addr, int port, String pwd ){
		this(addr,port);
		try {
			hostname = addr.getHostName();
		}catch (SecurityException e){
			hostname = addr.toString();
		};
		setPasswd(pwd);
	};
	public IBPDepot( String hostname, int port, String pwd ) throws
						UnknownHostException  {
		this(hostname,port);
		this.hostname = hostname;
		setPasswd(pwd);
	};
	
	
	
	// class fields
	String		hostname;
	InetAddress 	address;
	int	    	port;
	String		passwd = "ibp";
	
	//methods
	public void setPasswd( String pwd){
		if ( pwd.length() > IBPProtocol.DEPOT_PASSWD_LEN )
			passwd = pwd.substring(0,IBPProtocol.DEPOT_PASSWD_LEN-1);
	};
	
	public InetAddress getAddress() {
		return(address);
	};
	
	public int getPort(){
		return(port);
	};
	
	public String getHostName() {
		return(hostname);
	};
	
	public DepotStatus getStatus(String pwd, int cliTimeout, int svrTimeout ) 
						throws IOException, IBPException {
 		IBPSession session = new IBPSession(getAddress(),getPort(),cliTimeout);
		session.setTimeout(cliTimeout);
		String cmd = ""+IBPProtocol.IBP_v031+" "+IBPProtocol.IBP_STATUS+" "+
			     IBPProtocol.IBP_ST_INQ+" "+pwd+" "+svrTimeout+"\n";
		session.putCommUnit( new Request(cmd) );
		session.putCommUnit( new Response1());
		session.perform();
		String result = session.getResult();
		try {
			StringTokenizer parser = 
				new StringTokenizer(result,"\r\n ");
			String str = parser.nextToken();
			
			long stbStor = Long.parseLong(str);
			str = parser.nextToken();
			long stbStorUsed = Long.parseLong(str);
			str = parser.nextToken();
			long volStor = Long.parseLong(str);
			str = parser.nextToken();
			long volStorUsed = Long.parseLong(str);
			str = parser.nextToken();
			long dur = Long.parseLong(str);
			return( new DepotStatus(stbStor,stbStorUsed,volStor,volStorUsed,dur));	
		
		}catch (NumberFormatException e ){
			throw new IBPResponseFormatException();
		}catch ( NoSuchElementException e){
			throw new IBPResponseFormatException();
		}
			     
	};
	
	public DepotStatus getStatus(int cliTimeout, int svrTimeout ) 
					throws IOException, IBPException {
		return getStatus(passwd,cliTimeout,svrTimeout);			
	};				
	
	public DepotStatus getStatus(int svrTimeout ) 
					throws IOException, IBPException {
		return getStatus(passwd,0,svrTimeout);			
	};
	
	public DepotStatus getStatus( ) 
					throws IOException, IBPException {
		return getStatus(passwd,0,0);
	};
						
	public DepotStatus setLimit( String pwd, long std, long vol, long duration,
				int cliTimeout, int svrTimeout ) throws IOException,IBPException {
 		IBPSession session = new IBPSession(getAddress(),getPort(),cliTimeout);
		session.setTimeout(cliTimeout);
		String cmd = ""+IBPProtocol.IBP_v031+" "+IBPProtocol.IBP_STATUS+" "+
			     IBPProtocol.IBP_ST_CHANGE+" "+pwd+" "+svrTimeout+"\n";
		session.putCommUnit( new Request(cmd) );
		session.putCommUnit( new Response());
		cmd = ""+std+" "+vol+" "+duration+"\n";
		session.putCommUnit( new Request(cmd));
		session.putCommUnit( new Response1());
		session.perform();
		String result = session.getResult();
		try {
			StringTokenizer parser = 
				new StringTokenizer(result,"\r\n ");
			String str = parser.nextToken();
			
			long stbStor = Long.parseLong(str);
			str = parser.nextToken();
			long stbStorUsed = Long.parseLong(str);
			str = parser.nextToken();
			long volStor = Long.parseLong(str);
			str = parser.nextToken();
			long volStorUsed = Long.parseLong(str);
			str = parser.nextToken();
			long dur = Long.parseLong(str);
			return( new DepotStatus(stbStor,stbStorUsed,volStor,volStorUsed,dur));	
		
		}catch (NumberFormatException e ){
			throw new IBPResponseFormatException();
		}catch ( NoSuchElementException e){
			throw new IBPResponseFormatException();
		}
	
	};

	public DepotStatus setLimit( long std, long vol, long duration,
				int cliTimeout, int svrTimeout ) throws IOException,IBPException {
		return(setLimit(passwd,std,vol,duration,cliTimeout,svrTimeout));		
	};
	
	public DepotStatus setLimit( long std, long vol, long duration ) 
					throws IOException,IBPException {
		return(setLimit(passwd,std,vol,duration,0,0));				
	};
	
	public DepotStatus setLimit( String pwd, DepotStatus dptSt,
           int cliTimeout, int svrTimeout ) throws IOException,IBPException {
		return(setLimit(pwd,dptSt.getStbStorSize(),dptSt.getVolStorSize(),
			dptSt.getDuration(),cliTimeout,svrTimeout));
	};

	public DepotStatus setLimit(  DepotStatus dptSt,int cliTimeout, int svrTimeout ) 
	                                 throws IOException,IBPException {
		return(setLimit(passwd,dptSt,cliTimeout,svrTimeout));
	};

	public DepotStatus setLimit(  DepotStatus dptSt) throws IOException,IBPException {
	
		return(setLimit(passwd,dptSt,0,0));	
	};
	
	public static void main( String[] args){
		try{
			IBPDepot dpt = new IBPDepot("hydra4a.cs.utk.edu",11211);
			DepotStatus st = dpt.getStatus("IBP",3,0);
			System.out.println(""+st);
			DepotStatus st1 = dpt.setLimit("ibp",st.getStbStorSize()+500,st.getVolStorSize()+500,-1,0,0);
			System.out.println("After Change:");
			System.out.println(""+st1);
			st1.setVolStorSize(st1.getVolStorSize()+100);
			st1.setStbStorSize(st1.getStbStorSize()+100);
			st1 = dpt.setLimit(st1);
			System.out.println(""+st1);
		}catch ( UnknownHostException e){
			System.err.println("Unknown host!");
			System.exit(-1);
		}catch ( IBPTimeoutException e){
			System.err.println("Timeout occur!");
			System.exit(-1);
		}catch ( IOException e){
			System.err.println("IO ERROR!");
			System.exit(-1);
		}catch ( Exception e){
			System.err.println("ERRor!");
			System.exit(-1);
		};
		
		System.out.println("SUCCESS!!!");
				
	};
};

