package ibp;

import java.util.*;
import java.net.*;
import java.io.*;

//
// ManageCap.java
//
// Written by Yong Zheng

/**
  * This class presents a IBP Capability which has manage right  
  * 
  * 
  * @author   Yong Zheng
  */

public class ManageCap extends Capability {
	// constructor
	
	/**
	  * creates a read capability
	  *
	  * 	@param	cap	cap in string
	  */
	ManageCap( String cap ) throws IBPException, UnknownHostException {
		super(cap);
		if ( getType().compareTo("MANAGE") != 0 )
			throw new IBPAccessDeniedException();
	}	  
	  
	// methods
	/**
	  * incRefer increments the cap reference for specific type
	  *
	  *	@param	type	capability type 
	   *	@return NO return value
	  *	@exception throws IOException ,IBPException
	  */
	public void incRefer( int type ) 
					throws IOException,IBPException {
		if ( type != IBPProtocol.IBP_READCAP && 
		     type != IBPProtocol.IBP_WRITECAP )
		     	throw new IBPIllegalArgumentException();
			
		IBPSession session = 
				new IBPSession(getDepot(),getClientTimeout());
		String cmd = "" + IBPProtocol.IBP_v031 + " " +
			     IBPProtocol.IBP_MANAGE + " " +
			     getName() + " " +
			     getKey() + " " +
			     IBPProtocol.IBP_INCR + " " +
			     type + " " + 
			     getServerTimeout() + "\n";
		doTrans(session,cmd);	     
StringTokenizer parser = new StringTokenizer(session.getResult(),"\r\n ");
		try {
			String str = parser.nextToken();
			int retVal = Integer.parseInt(str.trim());
			if ( retVal != IBPProtocol.IBP_OK )
				throw IBPUtil.getException(retVal);
				
		}catch (NoSuchElementException e){
			throw new IBPResponseFormatException();
		}catch (NumberFormatException e){
			throw new IBPResponseFormatException();
		}	     
										
	}				
	
	/**
	  * decRefer decrements the cap reference for specific type
	  *
	  *	@param	type	capability type 
	  *	@return NO return value
	  *	@exception throws IOException ,IBPException
	  */
	public void decRefer( int type) 
					throws IOException,IBPException {
		
		if ( type != IBPProtocol.IBP_READCAP && 
		     type != IBPProtocol.IBP_WRITECAP )
		     	throw new IBPIllegalArgumentException();
		
		IBPSession session = 
				new IBPSession(getDepot(),getClientTimeout());
		String cmd = "" + IBPProtocol.IBP_v031 + " " +
			     IBPProtocol.IBP_MANAGE + " " +
			     getName() + " " +
			     getKey() + " " +
			     IBPProtocol.IBP_DECR + " " +
			     type + " " + 
			     getServerTimeout() + "\n";
		doTrans(session,cmd);
		StringTokenizer parser = new StringTokenizer(session.getResult(),"\r\n ");
		try {
			String str = parser.nextToken();
			int retVal = Integer.parseInt(str.trim());
			if ( retVal != IBPProtocol.IBP_OK )
				throw IBPUtil.getException(retVal);
				
		}catch (NoSuchElementException e){
			throw new IBPResponseFormatException();
		}catch (NumberFormatException e){
			throw new IBPResponseFormatException();
		}	     
								
	}
	
	/**
	  * getCapStatus checks the current state of the storage
	  * area accessed throught the cap
	  * 
	  * 	@param no parameter
	  *	@return CapStatus object
	  *	@exception throws IOException, IBPException
	  */				

	public CapStatus getCapStatus ( ) throws IOException,IBPException {
		IBPSession session = 
				new IBPSession(getDepot(),getClientTimeout());
		String cmd = "" + IBPProtocol.IBP_v031 + " " +
			     IBPProtocol.IBP_MANAGE + " " +
			     getName() + " " +
			     getKey() + " " +
			     IBPProtocol.IBP_PROBE + " " +
			     "1" + " " +  // position holder no use
			     "1" + " " +  // position holder no use
			     "1" + " " +  // position holder no use
			     "1" + " " +  // position holder no use 
			     getServerTimeout() + "\n";
		doTrans(session,cmd);
		StringTokenizer parser = new StringTokenizer(session.getResult(),"\r\n ");
		try {
			String str = parser.nextToken();
			int retVal = Integer.parseInt(str.trim());
			if ( retVal != IBPProtocol.IBP_OK )
				throw IBPUtil.getException(retVal);
			str = parser.nextToken();
			long rdRefCnt = Long.parseLong(str.trim());
			str = parser.nextToken();
			long wrRefCnt = Long.parseLong(str.trim());
			str = parser.nextToken();
			long curSize = Long.parseLong(str.trim());
			str = parser.nextToken();
			long maxSize = Long.parseLong(str.trim());
			str = parser.nextToken();
			long dur = Long.parseLong(str.trim());
			str = parser.nextToken();
			int rel = Integer.parseInt(str.trim());
			str = parser.nextToken();
			int tp = Integer.parseInt(str.trim());
			
			CapAttribute attr = new CapAttribute(dur,rel,tp);
			return new CapStatus(rdRefCnt,wrRefCnt,curSize,maxSize,attr);	
				
		}catch (NoSuchElementException e){
			throw new IBPResponseFormatException();
		}catch (NumberFormatException e){
			throw new IBPResponseFormatException();
		}
						
	}
	 

	/**
	  * changeCapStatus checks the current state of the storage
	  * area accessed throught the cap
	  * 
	  * 	@param status	holds the value to be set
	  *	@return no return
	  *	@exception throws IOException, IBPException
	  */				

	public void changeCapStatus ( CapStatus capSt ) throws IOException,IBPException {
		IBPSession session = 
				new IBPSession(getDepot(),getClientTimeout());
		String cmd = "" + IBPProtocol.IBP_v031 + " " +
			     IBPProtocol.IBP_MANAGE + " " +
			     getName() + " " +
			     getKey() + " " +
			     IBPProtocol.IBP_CHNG + " " +
			     "1" + " " +  // position holder no use
			     capSt.getMaxSize() + " " +
			     capSt.getCapAttr().getLifeTime() + " " + 
			     capSt.getCapAttr().getReliability() + " " +   
			     getServerTimeout() + "\n";
		doTrans(session,cmd);
		StringTokenizer parser = new StringTokenizer(session.getResult(),"\r\n ");
		try {
			String str = parser.nextToken();
			int retVal = Integer.parseInt(str.trim());
			if ( retVal != IBPProtocol.IBP_OK )
				throw IBPUtil.getException(retVal);
		}catch (NoSuchElementException e){
			throw new IBPResponseFormatException();
		}catch (NumberFormatException e){
			throw new IBPResponseFormatException();
		}	     
						
	}
	 

	private void doTrans( IBPSession session, String cmd)
			           throws IOException, IBPException {
		session.putCommUnit(new Request(cmd));
		session.putCommUnit(new Response1());
		session.perform();
	}  	  
}
