package ibp;

import java.util.*;
import java.net.*;
import java.io.*;

//
// WriteCap.java
//
// Written by Yong Zheng

/**
  * This class presents a IBP Capability which has write right  
  * 
  * 
  * @author   Yong Zheng
  */

public class WriteCap extends Capability {
	// constructor
	
	/**
	  * creates a read capability
	  *
	  * 	@param	cap	cap in string
	  */
	WriteCap( String cap ) throws IBPException, UnknownHostException {
		super(cap);
		if ( getType().compareTo("WRITE") != 0 )
			throw new IBPAccessDeniedException();
	};	  
	  
	// methods
	/**
	  * write writes size bytes starting at data in the 
	  * storage area accessed through the cap.
	  *
	  *	@param	buf	data buffer
	  *	@param	size	number of the bytes
	  *	@return number of bytes written
	  *	@exception throws IOException, IBPException
	  */
	public long write( byte[] buf, int size) throws IOException,IBPException {
		// check parameter
		if ( size <= 0 )
			throw new IBPIllegalArgumentException();
		// create a transaction
		IBPSession session = new IBPSession(getDepot(),getClientTimeout());
		// make a request
		String cmd = ""+IBPProtocol.IBP_v031+" "+
			     IBPProtocol.IBP_STORE+" "+
			     getName() + " "+
			     getKey() + " "+
			     size + " " +
			     getServerTimeout() + " \n";
		session.putCommUnit(new Request(cmd));
		session.putCommUnit(new Response());
		session.putCommUnit(new outData(buf,size));
		session.putCommUnit(new Response2());
		session.perform();
		return( session.getDataLen());	
					
	};				    
	  
};
