/**
 * ActiveHttpS.java
 *
 * @author Jean-Patrick Gelas
 */

/* service used for web data transport on udp and tcp */

/*
 french http proxy :
 ENS Lyon : http://www-cache.ens-lyon.fr:3128/
 UCBL     : http://proxy.univ-lyon1.fr:3128/

 net.raw_rd, net.raw_wr are used to communicate with the browser
 net_proxy.raw_rd, net_proxy.raw_wr with the proxy.
*/

import java.io.*;
import java.net.*     ;
import java.util.Date;

class ActiveHttpS extends Service 
{
    final static boolean DBG = true;

    final static String proxy_address = "www-cache.ens-lyon.fr";
    final static int proxy_port = 3128;

    // http headers
    final static String CONT_TYPE = "Content-Type:";
    final static String CONT_LENGTH = "Content-Length:";
    final static String CONT_ENCODING = "Content-Encoding:";
    
    char [] cbuf;
    final static int BUF_SIZE = 200 * 1024;

    
    final static String mesg_cant_reach_proxy = "<HTML><HEAD>"+
 	"<TITLE>Tamanoir Active Node</TITLE></HEAD>"+
 	"<BODY><H1>Tamanoir Active Node : Active Http Service</H1>"+
 	"Proxy -> " + proxy_address + " <- can not be reached."+
 	"</BODY></HTML>";


    ActiveHttpS() 
    {
	if(DBG) System.out.println( "ActiveHttpS : build" );
    }

    public void run()
    {
	// String req; 
	int   content_length = 0;
	TANet net_proxy=null;

	int c = 0, count;
	boolean is_text = true; /* text/html, text/plain,...*/

	try {
	    // 1 // connection to a real proxy 
	    Socket sp = new Socket(proxy_address, proxy_port);
	    net_proxy = new TANet(TANet.TCP, sp, false);
	    
	    // 2 // forward the browsers' request to the proxy
	    if(DBG) System.out.println("#from brwsr : " + net.mesg);
	    net_proxy.raw_wr.write( net.mesg + "\r\n");
	    String req;
	    while((req = net.raw_rd.readLine()) != null) {
		if(DBG) System.out.println("#from brwsr : " + req );
		net_proxy.raw_wr.write( req + "\r\n");
		if(req.equals("")) break;
	    }
	    net_proxy.raw_wr.flush();
	    
	    // 3 // read header and data from proxy
	    while((req = net_proxy.raw_rd.readLine()) != null) {
		if(DBG) System.out.println("#from proxy : " + req);

		if(req.startsWith(CONT_LENGTH)) {
		    String strLen = req.substring(CONT_LENGTH.length()).trim();
                    content_length = Integer.parseInt( strLen );
		    System.out.println("content_length = " + content_length);
		}
		if(req.startsWith(CONT_TYPE)) {
		    if(req.indexOf("text/") == -1) is_text = false;
		}
		if(req.startsWith(CONT_ENCODING)) { 
                    is_text = false;
                }

		net.raw_wr.write( req + "\r\n");
		if(req.equals("")) break;
	    }

	    count = 0;
	    switch (content_length) {
            case 0: // we don't know the size of the body
		cbuf = new char[BUF_SIZE];
		while(true) {
                    c = net_proxy.raw_rd.read(cbuf, count, BUF_SIZE - count);
                    if(c<=0) break; 
                    count += c;
		    if(DBG) System.out.println("c = " + c);
		}
		break;
	    default:
		cbuf = new char[content_length];
		while(count < content_length) {
		    c = net_proxy.raw_rd.read( cbuf, count, 
					       content_length - count );
		    if(DBG) System.out.println("c (proxy) = " + c);
		    if(c<0) break;
		    count += c;
		}
	    }
	    // 4 // send data to the browser
	    /***************/
	    if(is_text) process(); /****/
	    /***************/
	    net.raw_wr.write( cbuf, 0, count ); 
	    net.raw_wr.flush();
	    if(DBG) System.out.println("real Size= " + count );
	    
	} catch (UnknownHostException e) {
	    try {
		System.err.println("ActiveHttpS : Can't reach proxy");
		net.raw_wr.write("HTTP/1.0 200 OK\r\n"); // headers
		net.raw_wr.write("Date: " + new Date() + "\r\n");
		net.raw_wr.write("Server: "+"Tamanoir Active Node "+
				 "(Active Http Service)"+"\r\n");
		net.raw_wr.write("Content-length: " + 
				 mesg_cant_reach_proxy.length() + "\r\n");
		net.raw_wr.write("Content-type: "+"text/html"+"\r\n\r\n");
		net.raw_wr.write(mesg_cant_reach_proxy); // body
		net.raw_wr.flush();
		//net.close(); // mandatory for w3m, lynx.
	    } catch (IOException e1) {}
	} catch (IOException e) {
	    System.err.println( "ActiveHttpS : run() " + e );
        }
	if(DBG) System.out.println("close connections...");
	net_proxy.close(); // ???
	net.close(); // mandatory for w3m, lynx.

	if(DBG) System.out.println("ActiveHttpS : run() thread done.");
    }

    public void process()
    { // change lower-case letters in upper-case letters
	boolean in = false; // true if index is in a html tag 
        char c;
        for(int i=0; i < cbuf.length; i++) {
            c = cbuf[i];
            if(c == '<' || c == '&') in=true;
            if(!in) { 
		if(c >= 'a' && c<= 'z') // a..z
		    cbuf[i] = (char)(c-32);
	    } else {
                if(c == '>' || c == ';') in=false;
            }
        }
    } // process()


} // class ActiveHttpS
