/**
 *  GzipS.java
 *
 *  Compresses/Decompresses a byte array buffer 
 *
 *  @author Jean-Patrick Gelas
 */


import java.io.*;
import java.net.*;
import java.util.zip.*;


class GzipS extends Service 
{
    final static boolean DBG = true;
    byte[] bufout;

    GzipS() 
    {
	if(DBG) System.out.println( "GzipS : build" );
    }

    public void run()
    {
	try {
	    while(!FINISHED) {
		// compresses data in the GZIP format
		bufout = GZIPdata( net.getANEPkt().payload );
		double gain = 100.0 - (( 100.0 * bufout.length ) / (getANEPkt().payload.length ));
		if(DBG) System.out.println( "GzipS : gain = " + gain );
		net.makeANEPkt( net.serviceNameinBytes, 
				net.srcId.getBytes(),
				net.destId.getBytes(),
				net.lastId.getBytes(), 
				net.proxyTANId.getBytes(),
				bufout
				);
		net.sendANEPkt_next();

		recv();    // wait for the next ANEP packet
	    }
	} catch (IOException e) {
            System.err.println( "GzipS : run() " + e );
        }
	net.close();
	if(DBG) System.out.println("GzipS : run() thread done.");
    } // run()

    public void process()
    {
    }


    // GZIPdata : return a byte array of compressed data in the GZIP
    // format.
    public static byte [] GZIPdata( byte [] bufin )
	throws IOException
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	GZIPOutputStream gzos = new GZIPOutputStream(baos);

	gzos.write(bufin,0,bufin.length);
	gzos.close();
	return (baos.toByteArray());
    }

    // GUNZIPdata : returns quantity of decompressed bytes available
    // in bufout.  (we don't know the size of decompressed data before
    // decompressing data) (bufout must be oversized)
    public static int GUNZIPdata( byte [] bufin, byte [] bufout )
	throws IOException
    {
	ByteArrayInputStream bais = new ByteArrayInputStream( bufin );
	GZIPInputStream gunzipis = new GZIPInputStream( bais );

	int v = gunzipis.read( bufout, 0, bufout.length );
	gunzipis.close();
	return v;
    }



} // class GzipS



/*
public class C2 {

    static final int SIZE = 4096;
    static byte[] bufin = new byte[SIZE];
    static byte[] bufout;
    static byte[] bufin2;
    static byte[] bufout2 = new byte[2 * SIZE];

    public static void main(String[] args) {
      
	// init bufin
	for(int i=0; i<bufin.length;++i)
	    bufin[i] = (byte)i;  

	System.out.println( "START: bufin.length  = " + bufin.length);


	try {
	    // compresses data in the GZIP format
	    bufout = GZIPdata( bufin );
	    System.out.println( "[GZIP] bufout.length = " + bufout.length);

	    // decompresses data that has been stored in the GZIP format
	    bufin2 = bufout;
	    int v = GUNZIPdata( bufin2, bufout2 );
	    System.out.println( "[GUNZIP] decompr. bytes (v) = " + v);

	    // diff buffers -----------------------------
	    for(int i=0; i < v; i++)
		if(bufin[i] != bufout2[i]) 
		    System.out.print("ERROR");

	}
	//Simply print out any errors we encounter.
	catch (Exception e) {
	    System.err.println(e);
	}
    } // main()
    



    // return a byte array of compressed data in the GZIP format
    public static byte [] GZIPdata( byte [] bufin )
	throws IOException
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	GZIPOutputStream gzos = new GZIPOutputStream(baos);

	gzos.write(bufin,0,bufin.length);
	gzos.close();
	return (baos.toByteArray());
    }




    // returns quantity of decompressed bytes available in bufout.
    // (we don't know the size of decompressed data before decompressing data)
    // (bufout must be oversized)
    public static int GUNZIPdata( byte [] bufin, byte [] bufout )
	throws IOException
    {
	ByteArrayInputStream bais = new ByteArrayInputStream( bufin );
	GZIPInputStream gunzipis = new GZIPInputStream( bais );

	int v = gunzipis.read( bufout, 0, bufout.length );
	gunzipis.close();
	return v;
    }

}
*/


