/**
 *  Tcp2udpS.java
 *
 *  @author Jean-Patrick Gelas
 */

/* we suppose this kind of config */
/* (S)---tcp---[TAN1]->-udp->-[TAN2]---tcp---(R) */

/* supporte qu'un seul flux */


import java.io.*;
import java.net.*;

class Tcp2udpS extends Service 
{
    final static boolean DBG = true;

    byte [] localhostNameinBytes; // cache data for perf
    
    // TCP-2-UDP
    TANet netudp;
    boolean firstPkt = false;

    // UDP-2-TCP
    TANet nettcp_0 = null;
    
    Tcp2udpS() 
    {
	localhostNameinBytes = net.localhostName.getBytes();
	System.out.println("Tcp2udpS: net.localhostName = "+ net.localhostName);

    } // end of Tcp2udpS()

    public void run()
    {
	if(DBG) System.out.println("Tcp2udpS : Thread Start");
	
	// Tcp -> Udp : cree une socket udp
	if(net.proto == TANet.TCP) { 
	    InetAddress ia_nxtTAN = TAMANOIRd.route.routeToNode( net.proxyTANId );
	    if(ia_nxtTAN != null) {
		netudp = new TANet( TANet.UDP, ia_nxtTAN, TAMANOIRd.port, 0 );
		if(DBG) System.out.println( "Tcp2udpS : Udp socket builded" );
	    } else
		System.out.println("Tcp2udpS : ia_nxtTAN is null");
	    // Udp -> Tcp : cree une socket tcp
	} else { 
	    if(DBG) System.out.println( "Tcp2udpS : udp->tcp (nothing)" );
	}
	while(!FINISHED) {
	    try {
		// Tcp -> Udp // ------------------------------
		if(net.proto == TANet.TCP) { 
		    System.out.println("tcp2udp net = " + net );
		    // (approche naive) recupere le paquet ANEP et
		    // l'envoie sur un lien UDP. Pas d'ajout de numero
		    // de sequence.
		    if(firstPkt) {
			netudp.buildP2Ppath( net.proxyTANId, net.destId );
			firstPkt = false;
		    }
		    // now we can use 'netudp' to send data w/ udp transport
		    netudp.makeANEPkt( net.serviceNameinBytes, 
				       net.srcId.getBytes(),
				       net.destId.getBytes(),
				       localhostNameinBytes,
				       net.proxyTANId.getBytes(),
				       net.getANEPkt().payload );
		    netudp.sendANEPkt();
		}
		// Udp -> Tcp // ------------------------------
		else {
		    // Ce service sera appele a chaque fois qu'un
		    // paquet ANEP sur UDP arrivera (pas d'etat a
		    // priori !).  Pour le moment on suppose un seul
		    // flux (pas de gestion d'une table de hashage des
		    // connexions)
		    if( nettcp_0 == null ) {
 			nettcp_0 = new TANet( TANet.TCP, net.destId,
 					      TAMANOIRd.port );	    
 			// nettcp_0.isProxyTAN = true;
 			if(DBG) System.out.println( "Tcp2udpS : Tcp socket builded.");
 		    }
		    nettcp_0.makeANEPkt( net.serviceNameinBytes, 
					 net.srcId.getBytes(),
 					 net.destId.getBytes(),
 					 localhostNameinBytes,
 					 net.proxyTANId.getBytes(),
 					 net.getANEPkt().payload );
		    nettcp_0.sendPayload();
		    
		    if(DBG) System.out.println( "Tcp2udpS : tcp pkt sent !");
		}
		recv(); // wait for the next ANEP packet
	    } catch (IOException e) {
		System.err.println( "Tcp2udpS : run() : " + e );
	    }
	} // while (1)
	// net.close();
	//if(DBG) System.out.println("Tcp2udpS : Thread done");
    }
    
    public void process()
    {
    }
} // class Tcp2udpS




// 	} catch (BadANEPversionException e) {
// 	    System.err.println( "Tcp2udpS : recvANEPkt() : Received "+
// 				"unsupported ANEP version "+
// 				"exception : " + e );
// 	} catch (InterruptedException e) {
// 	    System.err.println("Tcp2udpS : timeout [>"+
		// 			       net.timeout+"]");



