/**
 * XcastIbpS.java
 *
 * @author Jean-Patrick Gelas
 */


import ibp.*;

import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Enumeration;

class XcastIbpS extends Service 
{
    final static boolean DBG = true;

    String last_destId = ""; 
    
    static final String IBP_DEPOT_NAME = "resamc"; // use a predefined depot
    static final int    IBP_DEPOT_PORT = 6716;
    static final int    CAP_SIZE = 1000 ; // KBytes 

    static final int TAMANOIR_Port = 5432;
    
    IBPDepot     dpt ;
    CapAttribute attr ;
    IBPCaps      caps ;
    
    WriteCap wc ;
    ReadCap  rc ;
    

    XcastIbpS() 
    {
	try{
            dpt = new IBPDepot( IBP_DEPOT_NAME ,IBP_DEPOT_PORT );
            attr = new CapAttribute(0,IBPProtocol.IBP_STABLE, 
				    IBPProtocol.IBP_BYTEARRAY);
            caps = new IBPCaps(dpt,CAP_SIZE*1024,attr,0,0); 
            wc = caps.getWriteCap(); 
            rc = caps.getReadCap();  
        } catch (UnknownHostException e) {
        } catch (IBPException e) {
	    System.err.println( "###->"+e);
        } catch (IOException e) {
        }
	if(DBG) System.out.println( "XcastIbpS : build" );
    }

    public void run()
    { 
	long ret1 = 0;

	while(!FINISHED) {
	    // 1/ cache payload in the local ibp depot 
	    try {
		ret1 = wc.write( getANEPkt().payload, 
				 getANEPkt().payload.length );
		System.err.println("ret1=" + ret1);
	    }catch (IOException e) {
		System.err.println("XcastIbpS 1: " + e);
	    }catch (IBPException e) {
		System.err.println("XcastIbpS 2: " + e);
	    }

	    // *** CLIENTS MANAGEMENT SECTION ***


	    // 2/ test if a new client is coming
	    if(!last_destId.equals(net.destId)) {
		try {
		    if(last_destId.equals("")) { // 1ST client
			new ClientS(net.wr_nxt, rc);
			if(DBG) System.out.println
				    ("XcastIbpS : 1st client ! ["+
				     net.destId+"]");
			last_destId = net.destId;
		    } else { // 2ND, 3TH, ... clients
			last_destId = net.destId;
			InetAddress ia_newCli = 
			    InetAddress.getByName(last_destId);
			Socket s = new Socket( ia_newCli, TAMANOIR_Port );
			DataOutputStream dos = 
			    new DataOutputStream(s.getOutputStream());
			new ClientS(dos, rc);
			if(DBG) System.out.println("XcastIbpS : New client !"+
						   " ["+ last_destId+"]");
		    }
		}catch(UnknownHostException e) {
		    System.err.println("XcastIbpS : " + e);
		}catch(IOException e) {
		    System.err.println("XcastIbpS : " + e);
		}
	    } // if(!last...

	    // 3/ wait for the next ANEP packet
	    recv();
	} // while(!FINISHED)

	//net.close();
	//last_destId = null;
	
	if(DBG) System.out.println("XcastIbpS : run() thread done.");
    } // run()

    public void process()
    {
	//if(DBG) net.display();
    }
} // class XcastIbpS



/* -----------------------------------------------------------------------*/

// class used by XcastIbpS
class ClientS extends Thread
{
    DataOutputStream dos ;
    ReadCap rc;
    long offset = 0;

    static final int SIZE = 512; // Bytes
    static final int DELAY = 500;
    byte [] buffer; 

    ClientS(DataOutputStream dos, ReadCap rc )
    {
	this.dos = dos;
	this.rc = rc;
	buffer = new byte[SIZE];
	start();
    }

    public void display() 
    {
	System.out.println("ClientS: " + dos + ", " + offset);
    }

    public void run()
    {
	int ret2;
	try {

	    do {
		ret2 = (int)rc.read(buffer, SIZE, offset);
		System.err.println("ret2=" + ret2);
		dos.writeInt(ret2);
		dos.write(buffer,0,ret2);
		offset += ret2;
		sleep(DELAY);
	    } while( ret2 > 0);

	} catch(IOException e) {
	    System.err.println("ClientS : " + e);
	} catch(InterruptedException e) {
	    System.err.println("ClientS : " + e);
	} catch (IBPException e) {
	    System.err.println("ClientS : " + e);
	}
	System.out.println("ClientS: Thread Done !");
    } // run()
	
} // class ClientS
