/**
 *  Pingping.java : throughput analysis tool
 *
 *  @author Jean-Patrick Gelas
 *  @email jpgelas@ens-lyon.fr
 *  @version 0.1
 */

import java.net.*;
import java.io.*;

public class Pingping extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = false;

    static final double INFINITE = 9999999L;
    static final int MAX_SIZE = 1024 * 64;
    static String serviceName = "FwdS"; //default service

    static String serviceBroker = "http://140.77.15.124"; 
    static int delay = 500; // default delay (ms) between each sent packet
    static int timeout = 1000; // default time out


    static String proxyTAN = null;
    static int proxyTANport = 5432; // default port

    static int proto = TANet.TCP; // default protocol is TCP
    static String destId = null;
    static int nbrPingPackets = 100;

    byte [] srvNameinBytes;
    byte [] srcIdinBytes  ;   
    byte [] destIdinBytes ;   
    byte [] lastIdinBytes ;   
    byte [] proxyTANIdinBytes;

    static int payloadSize = 4*1024; // default payload size (Bytes)
    static byte [] payload ;
    
    // UDP variables
    static DatagramSocket ds;

    TANet net;

    Pingping()
    {
    } 

    static void usage ()
    {
	System.out.println( "Pingping - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : Pingping [-h] [-size s] [-delay d] "+
			    "[-port p] [-to timeout]");
	System.out.println( "        [-udp | -tcp] [-n nbr] -tan proxyTAN "+
			    "-t target");
	System.out.println( "");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to "+
			    "a proxyTAN");
	System.out.println( "  -t target: ping pakets are sent towards "+
			    "target");
	System.out.println( "  -size s  : payload size of ANEP packets");
	System.out.println( "  -delay d : delay (ms) between each ping "+
			    "packet");
	System.out.println( "  -port p  : send data on port p of proxyTAN");


	System.out.println( "  -udp     : send data on a UDP channel");
	System.out.println( "  -tcp     : send data on a TCP channel "+
			    "(default)");
	System.out.println( "  -n nbr   : send nbr ping packets and exit");
	System.out.println();
	System.out.println( "ex: ..." );
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-size"))
		    {		
			i++;
			payloadSize = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-to"))
		    {		
			i++;
			timeout = Integer.parseInt(args[i]);;
		    }
		else if (args[i].equals ("-n"))
                    {
                        i++;
                        nbrPingPackets = Integer.parseInt(args[i]);
                    }
		else if (args[i].equals ("-t"))
		    {		
			i++;
			destId = args[i];
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name   : " + serviceName );
	System.out.println ("Service Broker : " + serviceBroker );
	System.out.println ("Delay          : " + delay );
	System.out.println ("proxy TAN      : " + proxyTAN );
	System.out.println ("port  TAN      : " + proxyTANport );
	System.out.println ("target         : " + destId);
        System.out.print   ("Protocole      : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
	System.out.println ("Payload size : " + payloadSize);
	System.out.println ("Timeout (ms) : " + timeout);
    }

    public int recv( DataInputStream rd )
    {
        int nbrPP = nbrPingPackets;
	int r_cpt=0;
        int c,size,count=0; // used by case TCP
	payload = new byte[MAX_SIZE];
        System.out.println("Waiting for "+nbrPingPackets+" packets");
        try {
            /* --- Recv 'nbrPingPackets' ANEP packets --- */
            while(nbrPP > 0) {
                switch(proto) {
                case TANet.TCP:
                    size = rd.readInt(); // read size of data incoming
                    if(DBG) System.err.println( "Pingping (read):size="+size);
                    count = 0;
                    while (count < size ) {
                        c = rd.read( payload, count, size - count );
                        if(c<0) break; // client socket has been closed
                        count += c;
                    }
                    if(DBG) System.err.println( "Pingping:recv (tcp):"+
						"reception completed [" + 
						count + " Bytes]" );
                    break;
                case TANet.UDP:
                    DatagramPacket dp = new DatagramPacket (payload, MAX_SIZE);
                    ds.receive (dp);
                    count = dp.getLength();
                    if(DBG) System.err.println( "Pingping:recv (udp):"+
						"reception completed ["+ 
						count + " Bytes]" );
                    break;
                } //switch()
                nbrPP--;
		r_cpt++;
            }
        } catch (IOException e) {
            System.err.println( "Pingping : recvANEPkt() [1]: " + e );
        }
	return r_cpt;
    }

    public int send_recvPingPackets()
    {
	// cache and init some data
	String sbsn = new String(serviceBroker + "/" + serviceName);
	srvNameinBytes    = sbsn.getBytes();
	srcIdinBytes      = net.srcId.getBytes();
	destIdinBytes     = destId.getBytes(); 
	lastIdinBytes     = srcIdinBytes;
	proxyTANIdinBytes = proxyTAN.getBytes(); 
	
	int t_cpt = 0, r_cpt=0;

	long pp_start = 0, pp_stop=10; // chrono pingping
	int nbrPP = nbrPingPackets;

	// build an ANEP packet
	net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
			destIdinBytes, lastIdinBytes, 
			proxyTANIdinBytes, payload );

	System.out.println("Pingping: sending "+nbrPingPackets+" packets...");
	pp_start = System.currentTimeMillis();
	try {
	    /* --- Send 'nbrPingPackets' ANEP packets --- */
	    while(nbrPP > 0) {
		net.sendANEPkt();	    
		t_cpt++;
		nbrPP--;
	    }
	    net.close();
	} catch (IOException e) {
            System.err.println( "Pingping : " + e );
        }


	/* --- Recv 'nbrPingPackets' ANEP packets --- */
	nbrPP = nbrPingPackets;
	System.out.println("Pingping: Waiting for "+nbrPingPackets+" packets");
	    try {
		switch(proto) {
		case TANet.TCP:
		    ServerSocket ss = new ServerSocket( proxyTANport );
		    if(DBG) System.err.println("Pingping : waiting for data "+
					       "on port " + proxyTANport);
		    Socket s = ss.accept();
		    if(s != null) {
			DataInputStream rd=
			    new DataInputStream(s.getInputStream());
			r_cpt = recv( rd );
		    } else 
			System.err.println("Pingping (tcp): problem !");
		    break;
		case TANet.UDP:
		    ds = new DatagramSocket ( proxyTANport );
		    if(ds != null) {
			r_cpt = recv( null );
		    } else
			System.err.println("Pingping (udp): problem !");
		    break;
		} // switch()
	    } catch (IOException e) {
		System.err.println( "Pingping : " + e );
	    }
	    pp_stop = System.currentTimeMillis();
	    
	    // display statistics results 
	    System.out.println( "--- " + destId + " Pingping statistics ---");
	    System.out.println( t_cpt + " packets transmitted, "+
				r_cpt + " packets received, "+(t_cpt-r_cpt)+
				" packets loss");
	    double pp_total = (pp_stop - pp_start) / 2.0;
	    double pkts = (1000 * nbrPingPackets) / pp_total;
	    double mbits = (nbrPingPackets * payloadSize * 8) / (pp_total * 1000);
	    System.out.println("Bandwidth = " + pkts + " Pkts/s,  " +
			       mbits + " Mbit/s");
	    return 0;
    } // send_recvPingPackets()


    public void run()
    {
	// Connect this thread to the closest proxyTAN
	// (given by arguments -tan hostname)
	switch(proto) {
        case TANet.TCP:
	    net = new TANet( TANet.TCP, proxyTAN, proxyTANport, timeout );
	    break;
	
	case TANet.UDP:
	    net = new TANet( TANet.UDP, proxyTAN, proxyTANport, timeout );
	    break;
	}
	
	// and finally send ping packets
	send_recvPingPackets();
	net.close();
	// if(DBG) System.out.println("Pingping : run() thread done.");
    } // run()
	

    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null || destId == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[payloadSize];
	for(int i=0; i<payloadSize; i++) payload[i] = 73;
	if(DBG) displayOptions();

	Pingping pt = new Pingping();
	pt.start(); // start thread

    } // main()
}

