/**
 *  Pingtan.java : TAN pinger 
 *
 *  @author Jean-Patrick Gelas
 *  @email jpgelas@ens-lyon.fr
 *  @version 0.1
 */

import java.net.*;
import java.io.*;

public class Pingtan extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = true;

    static final double INFINITE = 9999999L;
    
    static String serviceName = "PingS"; //default service

    static String serviceBroker = "http://140.77.15.124"; 
    static int delay = 500; // default delay (ms) between each sent packet
    static int timeout = 1000; // default time out
    static boolean infiniteLoop = true;

    static String proxyTAN = null;
    static int proxyTANport = 5432; // default port

    static int proto = TANet.TCP; // default protocol is TCP
    static String destId = null;
    static int nbrPingPackets = 0;
    // static String localAddress = null; // for DNS problem only

    byte [] srvNameinBytes;
    byte [] srcIdinBytes  ;   
    byte [] destIdinBytes ;   
    byte [] lastIdinBytes ;   
    byte [] proxyTANIdinBytes;

    static int payloadSize = 4*1024; // default payload size (Bytes)
    static byte [] payload ;
    
    static Toolbox tb;
    
    TANet net;

    static {
        System.loadLibrary("Toolbox");
    }

    Pingtan()
    {
    } 

    static void usage ()
    {
	System.out.println( "Pingtan - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : Pingtan [-h] [-size s] [-delay d] "+
			    "[-port p] [-to timeout]");
	System.out.println( "        [-udp | -tcp] [-n nbr] -tan proxyTAN "+
			    "-t target");
	System.out.println( "");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to "+
			    "a proxyTAN");
	System.out.println( "  -t target: ping pakets are sent towards "+
			    "target");
	System.out.println( "  -size s  : payload size of ANEP packets");
	System.out.println( "  -delay d : delay (ms) between each ping "+
			    "packet");
	System.out.println( "  -port p  : send data on port p of proxyTAN");


	System.out.println( "  -udp     : send data on a UDP channel");
	System.out.println( "  -tcp     : send data on a TCP channel "+
			    "(default)");
	System.out.println( "  -n nbr   : send nbr ping packets and exit");
	System.out.println( "  -la IP   : local adress (for DNS problem)");
	System.out.println( "ex: ..." );
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-size"))
		    {		
			i++;
			payloadSize = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-to"))
		    {		
			i++;
			timeout = Integer.parseInt(args[i]);;
		    }
		else if (args[i].equals ("-n"))
                    {
                        i++;
                        nbrPingPackets = Integer.parseInt(args[i]);
                    }
		else if (args[i].equals ("-t"))
		    {		
			i++;
			destId = args[i];
		    }
		else if (args[i].equals ("-la")) // use when DNS problem
		    {		
			i++;
			//localAddress = args[i]; 
			TAMANOIRd.frontendName = args[i]; 
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name   : " + serviceName );
	System.out.println ("Service Broker : " + serviceBroker );
	System.out.println ("Delay          : " + delay );
	System.out.println ("proxy TAN      : " + proxyTAN );
	System.out.println ("port  TAN      : " + proxyTANport );
	System.out.println ("target         : " + destId);
        System.out.print   ("Protocole      : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
	System.out.println ("Payload size : " + payloadSize);
	System.out.println ("Timeout (ms) : " + timeout);
    }


    public int sendPingPackets()
    {
	// cache and init some data
	String sbsn = new String(serviceBroker + "/" + serviceName);
	srvNameinBytes    = sbsn.getBytes();
	
	System.out.println("net.srcId = " + net.srcId);

	srcIdinBytes      = net.srcId.getBytes();
	destIdinBytes     = destId.getBytes(); 
	lastIdinBytes     = srcIdinBytes;
	proxyTANIdinBytes = proxyTAN.getBytes(); 
	
	int t_cpt = 0, r_cpt=0, timeout_cpt=0;

	long t1,t2=0; // chrono variables
	long pp_start = 0, pp_stop=10; // chrono pingping
	double t, total=0;
	int nbrPP = nbrPingPackets;
	double min = INFINITE , max = 0 ; // min, max round trip time
	
	if(nbrPingPackets != 0) infiniteLoop = false;

	pp_start = System.currentTimeMillis();

	while(nbrPP > 0 || infiniteLoop ) {
	    try {
		t1 = System.currentTimeMillis();
		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes, payload );
		if(delay == 0) t2 = System.currentTimeMillis();
		tb.startChrono_micros();
		net.sendANEPkt();	    
		t_cpt++;
		net.recvANEPkt();
		r_cpt++;

		tb.stopChrono_micros();
		if(delay != 0) t2 = System.currentTimeMillis();
		t = t2-t1;
		if(t < min) min = t;
		if(t > max) max = t;
		// if(t > timeout >> 1) max = 0;
		if(delay != 0) System.out.println( "Pingtan : #" + t_cpt +
						   " round-trip min/max = " + 
						   min + "/" + max + 
						   " ms   time= "+ t +" ms" );
		if(delay != 0) System.out.println("microsecondes = " + 
						  tb.getChrono_micros());


		if(!infiniteLoop) {
		    total += t;
		    nbrPP--;
		}
		if(delay != 0) {
		    try { sleep( delay ); } 
		    catch (InterruptedException e) {
		    }
		}
	    } catch (BadANEPversionException e) {
		System.err.println( "Pingtan : recvANEPkt() : Received " +
				    "unsupported ANEP version exception : "
				    +e);
	    } catch (IOException e) {
		System.err.println( "Pingtan : recvANEPkt() : " + e );
		timeout_cpt++;
		if(!infiniteLoop) nbrPP--;
	    } catch (InterruptedException e) {
		System.err.println("Pingtan : timeout [>"+timeout+"]");
		timeout_cpt++;
	    }
	} // while()

	pp_stop = System.currentTimeMillis();


	System.out.println( "--- " + destId + " Pingtan statistics ---");
	System.out.println( t_cpt + " packets transmitted, "+
			    r_cpt + " packets received, "+(t_cpt-r_cpt)+
			    " packets loss");
 	if(delay == 0) {
	    System.out.println( "Total time consuming by packets creation = " + total + " ms (is about of)");
	    double pp_total = (pp_stop - pp_start) / 2.0;
	    double pkts = (1000 * (nbrPingPackets - timeout_cpt)) / (pp_total - total);
	    double mbits = ((nbrPingPackets - timeout_cpt) * payloadSize * 8) / ((pp_total - total) * 1000);
	    System.out.println("Bandwidth = " + pkts + " Pkts/s,  " +
 			       mbits + " Mbit/s");
	} else {
	    if(min != INFINITE) {
		System.out.println( "round-trip min/avg/max = "+min+"/" + 
				    (total/t_cpt) +"/"+max +" ms");
	    }
	}
	System.out.println( timeout_cpt + " timeout occurs.");

	return 0;
    } // sendPingPackets()


    public void run()
    {
	// Connect this thread to the closest proxyTAN
	// (given by arguments -tan hostname)
	switch(proto) {
        case TANet.TCP:
	    net = new TANet( TANet.TCP, proxyTAN, proxyTANport, timeout );
	    break;
	
	case TANet.UDP:
	    net = new TANet( TANet.UDP, proxyTAN, proxyTANport, timeout );
	    break;
	}
	
	// and finally send ping packets
	sendPingPackets();
	net.close();
	// if(DBG) System.out.println("Pingtan : run() thread done.");
    } // run()
	

    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null || destId == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[payloadSize];
	for(int i=0; i<payloadSize; i++) payload[i] = 73; // 73 == 'I'
	if(DBG) displayOptions();

	tb = new Toolbox();
	
	Pingtan pt = new Pingtan();
	pt.start(); // start thread

    } // main()
}

