/**
 *  Reflector.java : throughput analysis tool (work w/ Pinping)
 *
 *  @author Jean-Patrick Gelas
 *  @email jpgelas@ens-lyon.fr
 *  @version 0.1
 */

import java.net.*;
import java.io.*;

public class Reflector extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = false;

    static final int MAX_SIZE = 1024 * 64;
    static byte [] payload = new byte[MAX_SIZE];
    
    static String serviceName = "FwdS"; //default service

    static String serviceBroker = "http://140.77.15.124"; 
    static int delay = 500; // default delay (ms) between each sent packet
    static int timeout = 1000; // default time out

    static String proxyTAN = null;
    static int proxyTANport = 5432; // default port

    static int proto = TANet.TCP; // default protocol is TCP
    static String destId = null;
    static int nbrPingPackets = 100;

    byte [] srvNameinBytes;
    byte [] srcIdinBytes  ;   
    byte [] destIdinBytes ;   
    byte [] lastIdinBytes ;   
    byte [] proxyTANIdinBytes;

    static int payloadSize = 4*1024; // default payload size (Bytes)

    // UDP variables
    static DatagramSocket ds;

    TANet net;

    Reflector()
    {
    } 

    static void usage ()
    {
	System.out.println( "Reflector - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : Reflector [-h] [-size s] [-delay d] "+
			    "[-port p] [-to timeout]");
	System.out.println( "        [-udp | -tcp] [-n nbr] -tan proxyTAN "+
			    "-t target");
	System.out.println( "");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to "+
			    "a proxyTAN");
	System.out.println( "  -t target: ping pakets are sent towards "+
			    "target");
	System.out.println( "  -size s  : payload size of ANEP packets");
	System.out.println( "  -delay d : delay (ms) between each ping "+
			    "packet");
	System.out.println( "  -port p  : send data on port p of proxyTAN");


	System.out.println( "  -udp     : send data on a UDP channel");
	System.out.println( "  -tcp     : send data on a TCP channel "+
			    "(default)");
	System.out.println( "  -n nbr   : send nbr ping packets and exit");
	System.out.println();
	System.out.println( "ex: ..." );
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-size"))
		    {		
			i++;
			payloadSize = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-to"))
		    {		
			i++;
			timeout = Integer.parseInt(args[i]);;
		    }
		else if (args[i].equals ("-n"))
                    {
                        i++;
                        nbrPingPackets = Integer.parseInt(args[i]);
                    }
		else if (args[i].equals ("-t"))
		    {		
			i++;
			destId = args[i];
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name   : " + serviceName );
	System.out.println ("Service Broker : " + serviceBroker );
	System.out.println ("Delay          : " + delay );
	System.out.println ("proxy TAN      : " + proxyTAN );
	System.out.println ("port  TAN      : " + proxyTANport );
	System.out.println ("target         : " + destId);
        System.out.print   ("Protocole      : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
	System.out.println ("Payload size : " + payloadSize);
	System.out.println ("Timeout (ms) : " + timeout);
    }


    public int recv_sendPingPackets(DataInputStream rd)
    {

	int nbrPP = nbrPingPackets;
	int c,size,count=0; // used by case TCP
	try {
	    /* --- Recv 'nbrPingPackets' ANEP packets --- */
	    while(nbrPP > 0) {
		switch(proto) {
                case TANet.TCP:
                    size = rd.readInt(); // read size of data incoming
                    if(DBG) System.err.println( "Reflector: recv size="+size);
                    count = 0;
                    while (count < size ) {
                        c = rd.read( payload, count, size - count );
                        if(c<0) break; // client socket has been closed
                        count += c;
                    }
                    if(DBG) System.err.println( "Reflector: recv (tcp):"+
						"reception completed ["+ 
						count + " Bytes]" );
                    break;
                case TANet.UDP:
                    DatagramPacket dp = new DatagramPacket (payload, MAX_SIZE);
                    ds.receive (dp);
                    count = dp.getLength();
                    if(DBG) System.err.println( "recvStream (udp): reception"+
                                                " completed [" + count +
                                                " Bytes]" );
                    break;
                } //switch()
		nbrPP--;
	    }
	    rd.close();
	} catch (IOException e) {
	    System.err.println( "Reflector : recv  [1]: " + e );
	}


	/* --- Send 'nbrPingPackets' ANEP packets --- */
	if(DBG) System.out.println("Sending "+nbrPingPackets+
				   " packets...");
	try {
	    long pp_start = 0, pp_stop=0; // chrono 
	    pp_start = System.currentTimeMillis();
	    // cache and init some data
	    switch(proto) {
	    case TANet.TCP:
		net = new TANet( TANet.TCP, proxyTAN, proxyTANport );
		break;
	    case TANet.UDP:
		net = new TANet( TANet.UDP, proxyTAN, proxyTANport );
		break;
	    }
	    String sbsn = new String(serviceBroker + "/" + serviceName);
	    srvNameinBytes    = sbsn.getBytes();
	    srcIdinBytes      = net.srcId.getBytes();
	    destIdinBytes     = destId.getBytes(); 
	    lastIdinBytes     = srcIdinBytes;
	    proxyTANIdinBytes = proxyTAN.getBytes();
	    
	    // build an ANEP packet
	    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
			    destIdinBytes, lastIdinBytes, 
			    proxyTANIdinBytes, payload );
	    if(DBG) System.out.println("ANEPakt builded");
	    pp_stop = System.currentTimeMillis();

	    System.out.println("Reflector: time for building ANEP pkt = "+
			       (pp_stop - pp_start) + " ms.");
	    nbrPP = nbrPingPackets;
	    while(nbrPP > 0) {
		net.sendANEPkt();	    
		nbrPP--;
	    }
	    net.close();
	} catch (IOException e) {
	    System.err.println( "Reflector : recvANEPkt() [2]: " + e );
	}
	return 0;
    } // recv_sendPingPackets()


    public void run()
    {
	ServerSocket ss = null;
	try {
	    while(true) {
		switch(proto) {
		case TANet.TCP:
		    ss = new ServerSocket( proxyTANport );
		    System.err.println("Reflector: waiting for "+
				       nbrPingPackets+ " packets on port "+
				       proxyTANport);
		    Socket s = ss.accept();
		    if(s != null) {
			DataInputStream rd=
			    new DataInputStream(s.getInputStream());
			recv_sendPingPackets( rd );
		    } else 
			System.err.println("Reflector (tcp): problem !");
		    break;
		case TANet.UDP:
		    ds = new DatagramSocket ( proxyTANport );
		    if(ds != null) {
			recv_sendPingPackets( null );
		    } else
			System.err.println("Reflector (udp): problem !");
		    break;
		} // switch()
		ss.close();
	    } // while(true)
	} catch (IOException e) {
	    System.err.println( "Reflector [3]: " + e );
	}
    } // run()
	

    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null || destId == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[payloadSize];
	for(int i=0; i<payloadSize; i++) payload[i] = 73;
	if(DBG) displayOptions();

	Reflector rf = new Reflector();
	rf.start(); // start thread

    } // main()
}

