/**
 * SendPackets.java : multithreaded stream server.
 *
 * @author Jean-Patrick Gelas
 */

import java.net.*;
import java.io.*;

public class SendPackets extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = false;
    
    static String serviceName = "FwdS"; // default service
    static int modulo = 100; // display info every 'modulo' sent packets
    static int delay  = 500; // default delay (ms) between each sent packet
    static String  proxyTAN = null;
    static int     proxyTANport = 5432; // default port
    static int     serverPort = 5432;   // default port
    static boolean fix = false;
    static int fix_size;
    // clients info
    String hostNameClient; // its name
    String proxyTANclient; // its proxyTAN
    int proxyTANportClient;// ...

    static int proto = TANet.TCP; // default protocol is TCP

    static String srcFileName = null ; // path for source file

    static int payloadSize = 4*1024; // default payload size
    static byte [] payload ;
    static final int SIZEOF_LONG = 8;

    static int numberOfStreams = 1;

    // TCP variables -----------------------------
    static ServerSocket ss;
    Socket cc;            // current connection
    TANet net;
    
    //Saad
    int num = 0;
    static int num_of_stream = 1;

    // TCP constructor
    SendPackets( Socket s, int num )
    {
        cc = s;
	//Saad
	this.num = num;
    } 



    static void usage ()
    {
	System.out.println( "SendPackets - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : SendPackets [-h] [-size s] [-delay d] "+
			    "[-port p] [-sn service]");
	System.out.println( "        [-udp | -tcp] [-src f|-fix s] -tan "+
			    "proxyTAN");
	System.out.println( "");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to a proxyTAN");
	System.out.println( "  -size s  : payload size of ANEP packets");
	System.out.println( "  -delay d : delay (ms) between each sent packet");
	System.out.println( "  -port p  : send data on port p of proxyTAN");
	System.out.println( "  -src f   : data are read in file f.");
	System.out.println( "  -fix s   : data are always the same (use always the same ANEP packet).");
	System.out.println( "             If -src or -fix are omitted use stdin");
	System.out.println( "  -udp     : send data on a UDP channel");
	System.out.println( "  -tcp     : send data on a TCP channel (default)");
	System.out.println( "  -sn name : service name (default is FwdS)");
	System.out.println( "  -m i     : display info every i pkts received");
	System.out.println( "  -ns n    : number of streams (default 1)");
        System.out.println( "  -n       : number of streams");
	System.out.println();
	System.out.println( "ex: ..." );
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-size"))
		    {		
			i++;
			payloadSize = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-sn"))
		    {		
			i++;
			serviceName = args[i];
		    }
		else if (args[i].equals ("-src") || args[i].equals ("-f"))
		    {		
			i++;
			srcFileName = args[i];
		    }
		else if (args[i].equals ("-fix"))
		    {	
			i++;	
			fix = true;
			fix_size = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-m"))
                    {
                        i++;
                        modulo = Integer.parseInt(args[i]);
                    }
		else if (args[i].equals ("-ns"))
                    {
                        i++;
                        numberOfStreams = Integer.parseInt(args[i]);
                    }
                else if (args[i].equals ("-n"))
                    {
                        i++;
                        num_of_stream = Integer.parseInt(args[i]);
                    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name : " + serviceName );
	System.out.println ("Delay        : " + delay );
	System.out.println ("proxy TAN    : " + proxyTAN );
	System.out.println ("port  TAN    : " + proxyTANport );
	System.out.print   ("source       : " );
	if(srcFileName == null) System.out.println("stdin");
	else System.out.println( srcFileName );
        System.out.print   ("Protocole    : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
	System.out.println ("Payload size : " + payloadSize);
    }


    //  sendFix
    //
    //  send always the same ANEP packet
    //
    //  5/11/2001: 64 first bits (long) of the payload are used as time-stamp
    //
    public int sendFix()
    {
	// cache some data
	byte [] srvNameinBytes    = serviceName.getBytes();
	byte [] destIdinBytes     = hostNameClient.getBytes(); 
	byte [] proxyTANIdinBytes = proxyTANclient.getBytes(); 

	byte [] srcIdinBytes      = net.srcId.getBytes();
        byte [] lastIdinBytes     = srcIdinBytes;

	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream( baos );

	try {
	    long t1,t2; // chrono variables
	    int cpt=0;
	    
	    for(int i=0; i<payloadSize;i++) payload[i] = 73;

	    int total_sz = fix_size; 
	    int sz = total_sz;
	    t1 = System.currentTimeMillis();
            while(sz > payloadSize) {

		if( cpt % modulo == 0 ) {
		    baos.reset();
		    dos.writeLong(System.currentTimeMillis());
		    System.arraycopy(baos.toByteArray(), 0, 
				     payload, 0, SIZEOF_LONG);
		}
		
		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes,
				payload );
		net.sendANEPkt();

		if( cpt++ % modulo == 0 )
		    System.out.println( "[# "+num+"]SendPackets : #" + cpt );
		sz -= payloadSize;
		if(delay != 0) try { 
		    Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}
	    } // while()

	    if(sz != 0) {
		byte [] t = new byte[sz];
		System.arraycopy(payload,0,t,0,sz);
		baos.reset();
		dos.writeLong(System.currentTimeMillis());
		System.arraycopy(baos.toByteArray(), 0, t, 0, SIZEOF_LONG);

		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes, t );
		net.sendANEPkt();
	    }
		
	    t2 = System.currentTimeMillis();
	    long t = t2 - t1;
	    System.out.println( "[# "+num+"]SendPackets : " + t + "ms for " +
				total_sz + "Bytes." );
	    System.out.println( "[# "+num+"]=> " + ((8*total_sz)/(1000*t))  
				+ " Mbits/sec." );
	    System.out.println( "[# "+num+"]=> " +  (((total_sz / payloadSize) * 1000) / t) 
				+ " packets/sec." );


	} catch (FileNotFoundException e ) { 
	    System.out.println("SendPackets : " + e ); 
	} catch (EOFException e ) { 
	    System.err.println("SendPackets : EOF '"+ srcFileName +"'"); 
	} catch (IOException e ) { 
	    System.err.println("SendPackets : " + e );
	    System.exit(0);
	}
	return 0;
    } // sendFix()

    public void run()
    {
	try {
	    // recover some informations from client (its name,
	    // proxyTANname where it is connected, and port)
	    DataInputStream rd_cli = new DataInputStream(cc.getInputStream());

	    hostNameClient = rd_cli.readUTF();
	    proxyTANclient = rd_cli.readUTF();
	    proxyTANport   = rd_cli.readInt();
	    
	    if(DBG) System.out.println("[# "+num+"]SendPackets : (new cnx) client:"+
				       hostNameClient+" proxyTAN:"+
				       proxyTANclient+" ("+
				       proxyTANport+")"
				       );
	    rd_cli.close();
	    cc.close(); // we don't use anymore this connection.
	} catch (IOException e) {
	    System.err.println( "SendPackets (tcp): " + e );
	}
	
	
	// Now we have to connect this thread to the closest proxyTAN
	// (given by arguments -tan hostname)
	switch(proto) {
        case TANet.TCP:
	    net = new TANet( TANet.TCP, proxyTAN, proxyTANport );
	    break;
	
	case TANet.UDP:
	    net = new TANet( TANet.UDP, proxyTAN, proxyTANport );
	    break;
	}
	
	// and finally send data...
	if(fix) 
	    sendFix();
	
	net.close();
	if(net.dsr != null) net.dsr.close();
	if(DBG) System.out.println("[# "+num+"]SendPackets : run() thread done.");
    } // run()
    
    
    public static void main( String [] args )
    {
	parseArgs( args );
	if(proxyTAN == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[payloadSize];
	if(DBG) displayOptions();

	// wait for new clients
	try {
	    ss = new ServerSocket( serverPort );
	    //	    while( true ) {
	    //Saad
	    for(int i=0; i < num_of_stream; i++) { 
		System.out.println("SendPackets (tcp): waiting for connection"+
				   " on port " + serverPort);
		SendPackets sendp = new SendPackets( ss.accept(), i);
		System.err.println("sender accept...");
		sendp.start(); // start thread
	    }
	    //	    }
	} catch (IOException e) {
	    System.err.println( "SendPackets : " + e );
	}
    } // main()
}

