/**
 *  TAMonitor.java : monitor of TAManoir node
 *
 *  @author Jean-Patrick Gelas
 *  @email jpgelas@ens-lyon.fr
 *  @version 0.1
 */

import java.net.*;
import java.io.*;

public class TAMonitor extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = true;

    static String serviceName = "MonitorS"; //default service

    static String serviceBroker = "http://140.77.15.124"; 
    static int delay = 2000; // default delay (ms) between each sent packet


    static boolean infiniteLoop = true;

    static String proxyTAN = null;
    static int proxyTANport = 5432; // default port
    static int serverPort = 5432;   // default port

    static int proto = TANet.TCP; // default protocol is TCP
    static String destId = null;

    byte [] srvNameinBytes;
    byte [] srcIdinBytes  ;   
    byte [] destIdinBytes ;   
    byte [] lastIdinBytes ;   
    byte [] proxyTANIdinBytes;

    static byte [] payload ;
    final static int INT_SIZE = 4; // (Bytes) 32 bits signed integer

    TANet net;

    // variables for statistics use
    static long recv_udp_counter = 0; // number of received packets
    static long send_udp_counter = 0; // number of sent packets
    static long outputsize_udp_counter = 0; // data size tranmitted
    static long inputsize_udp_counter = 0; // data size received

    static long recv_tcp_counter = 0; // number of received packets
    static long send_tcp_counter = 0; // number of send packets
    static long outputsize_tcp_counter = 0; // data size tranmitted
    static long inputsize_tcp_counter = 0; // data size received

    TAMonitor()
    {
    } 

    static void usage ()
    {
	System.out.println( "TAMonitor - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : TAMonitor [-h] [-delay d] "+
			    "[-port p] -tan proxyTAN -t target");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to "+
			    "a proxyTAN");
	System.out.println( "  -t target: monitored TAN");
	System.out.println( "  -delay d : refresh info frequency (ms)");
	System.out.println( "  -port p  : send data on port p of proxyTAN");
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-t"))
		    {		
			i++;
			destId = args[i];
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name   : " + serviceName );
	System.out.println ("Service Broker : " + serviceBroker );
	System.out.println ("Delay          : " + delay );
	System.out.println ("proxy TAN      : " + proxyTAN );
	System.out.println ("port  TAN      : " + proxyTANport );
	System.out.println ("target         : " + destId);
        System.out.print   ("Protocole      : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
    }


    public int sendDelayPackets()
    {

	long t0 = 0;
	long old_inputsize_udp_counter = 0;
	long old_outputsize_udp_counter = 0;
	long old_inputsize_tcp_counter = 0; 
	long old_outputsize_tcp_counter = 0; 

	// cache and init some data
	String sbsn = new String(serviceBroker + "/" + serviceName);
	srvNameinBytes    = sbsn.getBytes();
	srcIdinBytes      = net.srcId.getBytes();
	destIdinBytes     = destId.getBytes(); 
	lastIdinBytes     = srcIdinBytes;
	proxyTANIdinBytes = proxyTAN.getBytes(); 
	
	try {
	    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
			    destIdinBytes, lastIdinBytes, 
			    proxyTANIdinBytes, payload );
	    net.sendANEPkt();	    

	    while( true ) {
		net.recvANEPkt();
		net.recoverInfoFromANEPkt();
		ByteArrayInputStream bais = 
		    new ByteArrayInputStream( net.getANEPkt().payload );
		DataInputStream dis = new DataInputStream( bais );
		System.out.println("Info from : [ " + net.srcId + 
				   " ] ------");
		
		recv_udp_counter       = dis.readLong(); 
   		inputsize_udp_counter  = dis.readLong(); 
		send_udp_counter       = dis.readLong(); 
		outputsize_udp_counter = dis.readLong();

		recv_tcp_counter       = dis.readLong(); 
		inputsize_tcp_counter  = dis.readLong();
		send_tcp_counter       = dis.readLong();
		outputsize_tcp_counter = dis.readLong();

		long t_now = System.currentTimeMillis();

		if( t0 != 0) {
		    System.out.println("UDP :recv " + recv_udp_counter + " ["+ inputsize_udp_counter +"]" +" "+ (((inputsize_udp_counter - old_inputsize_udp_counter) * 8) / (t_now - t0)) +" Kbit/s");
		    
		    System.out.println("     send " + send_udp_counter + " ["+ outputsize_udp_counter +"]" +" "+ (((outputsize_udp_counter - old_outputsize_udp_counter) * 8) / (t_now - t0))+" Kbit/s");
		    
		    System.out.println("TCP :recv " + recv_tcp_counter  + " ["+ inputsize_tcp_counter +"]" +" "+ (((inputsize_tcp_counter - old_inputsize_tcp_counter ) * 8) / (t_now - t0)) +" Kbit/s");
		    
		    System.out.println("     send " + send_tcp_counter  + " ["+ outputsize_tcp_counter +"]" +" "+ (((outputsize_tcp_counter - old_outputsize_tcp_counter ) * 8) / (t_now - t0)) +" Kbit/s");
		}
		t0 = t_now;
		old_inputsize_udp_counter = inputsize_udp_counter;
		old_outputsize_udp_counter = outputsize_udp_counter;
		old_inputsize_tcp_counter = inputsize_tcp_counter;
		old_outputsize_tcp_counter = outputsize_tcp_counter;
	    } // while( true )
	} catch (BadANEPversionException e) {
	    System.err.println( "TAMonitor : recvANEPkt() : Received " +
				"unsupported ANEP version exception : "
				+e);
	} catch (IOException e) {
	    System.err.println( "TAMonitor : recvANEPkt() : " + e );
	} catch (InterruptedException e) {
	    System.err.println("TAMonitor : " + e );
	}
	return 0;
    } // sendDelayPackets()


    public void run()
    {
	// ????
	// Connect this thread to the closest proxyTAN
	// (given by arguments -tan hostname)
	// ????
	net = new TANet( TANet.TCP, proxyTAN, proxyTANport );

	// and finally send ping packets
	sendDelayPackets();
	net.close();
	// if(DBG) System.out.println("TAMonitor : run() thread done.");
    } // run()
	

    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null || destId == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[1*INT_SIZE];
	try {
	    ByteArrayOutputStream baos = 
		new ByteArrayOutputStream(1*INT_SIZE);
	    DataOutputStream dos = new DataOutputStream( baos );
	    dos.writeInt( delay );
	    payload = baos.toByteArray();
	    if(DBG) displayOptions();
	} catch (IOException e) {
	    System.err.println("TAMonitor : " + e);
	}
	TAMonitor tm = new TAMonitor();
	tm.start(); // start thread

    } // main()
}

