/**
 * VoDS.java : Video on Demand Server
 * (multithreaded stream server)
 *
 * @author Jean-Patrick Gelas
 */

import java.net.*;
import java.io.*;

public class VoDS extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = true;
    
    static String serviceName = "XcastS"; // default service
    static int modulo = 100; // display info every 'modulo' sent packets
    static int delay  = 500; // default delay (ms) between each sent packet
    static String proxyTAN = null;
    static int proxyTANport = 5432; // default port
    static int serverPort = 5432;   // default port
    static boolean fix = false;
    static int fix_size;
    // clients info
    String hostNameClient; // its name
    String proxyTANclient; // its proxyTAN
    int proxyTANportClient;// ...

    static int proto = TANet.TCP; // default protocol is TCP

    static String srcFileName = null ; // path for source file
    static boolean firstClient = true; // true until someone connect server

    // cache some data
    static byte [] srvNameinBytes;
    static byte [] destIdinBytes ;
    static byte [] proxyTANIdinBytes ;
    static byte [] srcIdinBytes ;
    static byte [] lastIdinBytes;

    static int payloadSize = 4*1024; // default payload size
    static byte [] payload ;

    // TCP variables -----------------------------
    static ServerSocket ss;
    Socket cc;            // current connection
    static TANet net;


    // TCP constructor
    VoDS( Socket s )
    {
        cc = s;
    } 



    static void usage ()
    {
	System.out.println( "VoDS - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : VoDS [-h] [-size s] [-delay d] "+
			    "[-port p] [-sn service]");
	System.out.println( "        [-udp | -tcp] [-src f|-fix s] -tan "+
			    "proxyTAN");
	System.out.println( "");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to a proxyTAN");
	System.out.println( "  -size s  : payload size of ANEP packets");
	System.out.println( "  -delay d : delay (ms) between each sent packet");
	System.out.println( "  -port p  : send data on port p of proxyTAN");
	System.out.println( "  -src f   : data are read in file f.");
	System.out.println( "  -fix s   : data are always the same (use always the same ANEP packet).");
	System.out.println( "             If -src or -fix are omitted use stdin");
	System.out.println( "  -udp     : send data on a UDP channel");
	System.out.println( "  -tcp     : send data on a TCP channel (default)");
	System.out.println( "  -sn name : service name (default is "+serviceName+")");
	System.out.println( "  -m i     : display info every i pkts received");
	System.out.println();
	System.out.println();
	System.out.println( "ex: ..." );
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-size"))
		    {		
			i++;
			payloadSize = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-sn"))
		    {		
			i++;
			serviceName = args[i];
		    }
		else if (args[i].equals ("-src") || args[i].equals ("-f"))
		    {		
			i++;
			srcFileName = args[i];
		    }
		else if (args[i].equals ("-fix"))
		    {	
			i++;	
			fix = true;
			fix_size = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-m"))
                    {
                        i++;
                        modulo = Integer.parseInt(args[i]);
                    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name : " + serviceName );
	System.out.println ("Delay        : " + delay );
	System.out.println ("proxy TAN    : " + proxyTAN );
	System.out.println ("port  TAN    : " + proxyTANport );
	System.out.print   ("source       : " );
	if(fix) System.out.println("fixed ["+ fix_size+"]");
	else if(srcFileName == null) System.out.println("stdin");
	else System.out.println( srcFileName );
        System.out.print   ("Protocole    : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
	System.out.println ("Payload size : " + payloadSize);
    }

    public int sendFile() 
    {
	// cache some data
	srvNameinBytes    = serviceName.getBytes();
	proxyTANIdinBytes = proxyTANclient.getBytes(); 

	srcIdinBytes      = net.srcId.getBytes();
        lastIdinBytes     = srcIdinBytes;

	// proxyTANportClient;
	
	try {
	    long t1,t2; // chrono variables
	    int cpt=0;
	    BufferedInputStream in = new BufferedInputStream(new FileInputStream(srcFileName), payloadSize);
	    int sz = 0, total_sz = 0;
	    t1 = System.currentTimeMillis();
            while((sz = in.read(payload)) > 0) {
		if(sz == payloadSize) {
		    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				    destIdinBytes, lastIdinBytes, 
				    proxyTANIdinBytes,
				    payload );
		} else {
		    byte [] t = new byte[sz];
		    System.arraycopy(payload,0,t,0,sz);
		    if(DBG) System.out.println("VoDS : Arghhh !!!");
		    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				    destIdinBytes, lastIdinBytes, 
				    proxyTANIdinBytes, t );
		}
		net.sendANEPkt();

		if( cpt++ % modulo == 0 )
		    System.out.println( "VoDS : #" + cpt );
		total_sz += sz;
		try { Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}
	    } // while()
	    t2 = System.currentTimeMillis();
	    System.out.println( "VoDS : " + (t2-t1) + "ms for " +
				total_sz + " Bytes." );

	} catch (FileNotFoundException e ) { 
	    System.out.println("VoDS : " + e ); 
	} catch (EOFException e ) { 
	    System.err.println("VoDS : EOF '"+ srcFileName +"'"); 
	} catch (IOException e ) { 
	    System.err.println("VoDS : " + e );
	    System.exit(0);
	}
	return 0;
    } // sendFile()



    public int sendStdin()
    // data source is stdin
    {
	// cache some data
	srvNameinBytes    = serviceName.getBytes();
	proxyTANIdinBytes = proxyTANclient.getBytes(); 

	srcIdinBytes      = net.srcId.getBytes();
        lastIdinBytes     = srcIdinBytes;
		
	try {
	    int cpt=0;
	    BufferedInputStream in = new BufferedInputStream(System.in);
	    int sz = 0;
	    while((sz = in.read(payload)) > 0) {
		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes, 
				payload );
		net.sendANEPkt();
		
		if(DBG) if( cpt++ % modulo == 0 ) 
		    System.err.println( "VoDS : #" + cpt );
		
		try { Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}
	    } // while()
	} catch (IOException e) {
	    System.err.println("VoDS (sendStdin) : " + e);
	}
	return 0;
    } // sendStdin()


    public int sendFix()
    // send always the same ANEP packet
    {
	// cache some data
	srvNameinBytes    = serviceName.getBytes();
	proxyTANIdinBytes = proxyTANclient.getBytes(); 

	srcIdinBytes      = net.srcId.getBytes();
        lastIdinBytes     = srcIdinBytes;

	try {
	    long t1,t2; // chrono variables
	    int cpt=0;
	    
	    for(int i=0; i<payloadSize;i++) payload[i] = 73;
	    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
			    destIdinBytes, lastIdinBytes, 
			    proxyTANIdinBytes,
			    payload );

	    int total_sz = fix_size; 
	    int sz = total_sz;
	    t1 = System.currentTimeMillis();
            while(sz > payloadSize) {

		net.sendANEPkt();

		if( cpt++ % modulo == 0 )
		    System.out.println( "VoDS : #" + cpt );
		sz -= payloadSize;
		try { Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}
	    } // while()
	    if(sz != 0) {
		byte [] t = new byte[sz];
		System.arraycopy(payload,0,t,0,sz);
		if(DBG) System.out.println("VoDS : Arghhh !!!");
		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes, t );
		net.sendANEPkt();
	    }
		
	    t2 = System.currentTimeMillis();
	    System.out.println( "VoDS : " + (t2-t1) + "ms for " +
				total_sz + " Bytes." );

	} catch (FileNotFoundException e ) { 
	    System.out.println("VoDS : " + e ); 
	} catch (EOFException e ) { 
	    System.err.println("VoDS : EOF '"+ srcFileName +"'"); 
	} catch (IOException e ) { 
	    System.err.println("VoDS : " + e );
	    System.exit(0);
	}
	return 0;
    } // sendFix()


    public void run()
    {
	try {
	    // recover some informations from client (its name,
	    // proxyTANname where it is connected, and port)
	    DataInputStream rd_cli = new DataInputStream(cc.getInputStream());

	    hostNameClient = rd_cli.readUTF();
	    proxyTANclient = rd_cli.readUTF();
	    proxyTANport   = rd_cli.readInt();
	    
	    if(DBG) System.out.println("VoDS : (new cnx) client:"+
				       hostNameClient+" proxyTAN:"+
				       proxyTANclient+" ("+
				       proxyTANport+")"
				       );
	    rd_cli.close();
	    cc.close(); // we don't use anymore this connection.
	} catch (IOException e) {
	    System.err.println( "VoDS (tcp): " + e );
	}
	
	if(firstClient) {

	    System.out.println( "VoDS : firstClient (1)" );
	    
	    // Now we have to connect this thread to the closest proxyTAN
	    // (given by arguments -tan hostname)
	    switch(proto) {
	    case TANet.TCP:
		net = new TANet( TANet.TCP, proxyTAN, proxyTANport );
		break;
	    case TANet.UDP:
		net = new TANet( TANet.UDP, proxyTAN, proxyTANport );
		break;
	    }
	}

	// change the destId field with new hostNameClient to add it in
	// the list hold by the Xcast*S (service)
	destIdinBytes = hostNameClient.getBytes(); 

	if(firstClient) {
	    System.out.println( "VoDS : firstClient (2)" );
	    firstClient = false;
	    // and finally send data... (only for the first client)
	    if(srcFileName != null) {
		System.out.println("send File...");
		sendFile();
	    } else {
		if(fix) {
		    System.out.println("send Fix...");
		    sendFix();
		} else
		    sendStdin();
	    }
	    firstClient = true;
	    net.close();
	    // if(net.ds != null) net.ds.close();
	}
	    
	    
	
	if(DBG) System.out.println("VoDS : run() thread done.");
    } // run()
	

    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[payloadSize];
	if(DBG) displayOptions();

	// wait for new clients
	try {
	    ss = new ServerSocket( serverPort );
	    System.out.println("VoDS (tcp): waiting for connection"+
			       " on port " + serverPort);
	    while( true ) {
		VoDS sendp = new VoDS( ss.accept() );
		sendp.start(); // start thread
	    }
	} catch (IOException e) {
	    System.err.println( "VoDS : " + e );
	}
    } // main()
}

