/**
 *  class ServiceManager
 *
 *
 *  @author Jean-Patrick Gelas
 */

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

import java.io.*;
import java.net.*;


public class ServiceManager {

    static boolean DBG = false ; // verbose mode if true

    final static int NUMBER_MAX_OF_SERVICE = 50;
    static int numberOfService = 0;
    static Hashtable services = new Hashtable();

    /* Return number Of Service */
    int number() {
	return numberOfService;
    }


    /* Return service */
    Service get( String id ) {
	ServiceDefinition sd = (ServiceDefinition)services.get( id );
	sd.incHint();
	return sd.getService();
    }
    
    /* Download a service from a service broker (http server) */
    void download( String serviceName )
    {
	String sn = new String(serviceName);
	if(sn.startsWith("http:")) {
	    StringTokenizer stok = new StringTokenizer(sn,"/");
	    stok.nextToken(); // http:	
	    stok.nextToken(); // 140.77.15.124
	    sn = stok.nextToken();
	    if(DBG) System.out.println("Service Manager: sn = " + sn);
	}
	// check first if serviceName is not available on local disk
	File f = new File(TAMANOIRd.service_path + sn + 
			  TAMANOIRd.shared_obj_ext);


	if(!f.canRead()) {
	    try {
		System.out.println("Loading "+serviceName+"service from "+TAMANOIRd.serviceBroker);
		URL u = new URL(TAMANOIRd.serviceBroker+"/"+serviceName+TAMANOIRd.shared_obj_ext);
		URLConnection uc = u.openConnection();
		String ct = uc.getContentType();
		int cl = uc.getContentLength();
		//if(ct.startsWith("text/") || cl==-1) {
		//System.err.println("ServiceManager:download():"+
		//"Not a binary file, cl=" + cl);
		// return;
		//}
		InputStream is = uc.getInputStream();
		byte [] b = new byte[cl];
		if(DBG) System.out.println("ServiceManager: cl = " + cl);
		int count = 0;
		int c = 0;
		while(count < cl) {
		    c = is.read(b, count, cl-count);
		    if(DBG) System.out.println("ServiceManager:download(): "+
					       "count = " + count + 
					       ", c = " + c);
		    if(c<0) break;
		    count += c;
		}
		if(count != cl) {
		    System.err.println("ServiceManager:download():"+
				       "Error: Only "+ count+" bytes read");
		    System.err.println("but "+cl+" should be read.");
		}
		String currentFile = u.getFile();
		currentFile=currentFile.substring(currentFile.lastIndexOf("/")+1);
		if(DBG) System.out.println("currentFile = " + currentFile);
		FileOutputStream fout = new FileOutputStream(currentFile);
		fout.write(b);
		fout.close();
	    } catch (FileNotFoundException e) {
		System.err.println( "ServiceManager:download(): " + e );
	    } catch (IOException e) {
		System.err.println( "ServiceManager:download(): " + e );
	    }
	} 
	else {
	    if(DBG) System.out.println("ServiceManager:download(): " +
				       sn + " already available " +
				       "on local disk");
	} // if(!f.canRead())

	if(!TAMANOIRd.RUN_IN_JVM) {
	    if(DBG) System.out.println("ServiceManager: LSO");
	    LSO lso = new LSO ();
	    lso.loadSharedObject ( TAMANOIRd.service_path + sn + 
				   TAMANOIRd.shared_obj_ext );
	}
	

    } // download()


    /* Search for a service availability in hash table */
    boolean contains( String id ) {
	return services.containsKey( id );
    }
    
    /* Delete a service */
    void delete( String id ) {
	if( contains( id ) ) {
	    services.remove( id );    
	    numberOfService--;
	}
    }
    
    /* find the first older service, return index */
    String findOlder() {
	long older = System.currentTimeMillis();
	String olderKey = "";
	
	for(Enumeration e = services.keys(); e.hasMoreElements(); ) {
	    String key = (String)e.nextElement();
	    ServiceDefinition elem = (ServiceDefinition)services.get( key );
      
	    if( older > elem.getBirth() ) {
		older = elem.getBirth();
		olderKey = key;
	    }
	}
	return olderKey;
    }
    
    
    /* Delete the Last Recently Used (LRU) service */
    void deleteLRU() {
	delete( findOlder() );
    }
    
    
    int add( String id , Service newService ) {
	if( numberOfService == NUMBER_MAX_OF_SERVICE ) return 0; 
	services.put( id, new ServiceDefinition( newService ) );
	numberOfService++;
	return numberOfService;
    }
    
    void display() 
    {
	int num = 1;
	System.out.println( "Number of service(s) : " + numberOfService );
	
	for( Enumeration e = services.keys() ; e.hasMoreElements(); num++ ) {
	    String key = (String)e.nextElement();
	    ServiceDefinition elem = (ServiceDefinition)services.get( key );	
	    long old_s = elem.getAge_ms() / 1000 ;
	    System.out.println( num + ") " + key + " : " + elem.getHint() + 
				" times used, " + old_s + " old seconds" );
	}
    } // display()


} // class ServiceManager



