/* ***************************************************************
 * TAMANOIR :  A High Performance Active Network Framework       *
 *                        Laboratoire ReSAM                      *
 *              Universite Claude Bernard, Lyon I, France        *
 *                       Action INRIA - RESO                     *
 *            Authors:  Jean-Patrick Gelas, Laurent Lefevre      *
 *                   (C) 2000 All Rights Reserved                *
 * ***************************************************************

                             NOTICE

 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted
 provided that the above copyright notice appear in all copies and
 that both the copyright notice and this permission notice appear in
 supporting documentation.

 Neither the institutions (Ecole Normale Superieure de Lyon,
 Universite Claude Bernard - Lyon I), nor the Authors make any
 representations about the suitability of this software for any
 purpose. This software is provided ``as is'' without express or
 implied warranty.

 Academic use authorized. Commercial use forbidden.  Please freely
 register by sending an email to <jpgelas@www.ens-lyon.fr> to
 receive updates and informations.

 Tamanoir was funded in part by ANVAR, INRIA.
 * ***************************************************************/

/**
 *  TAMANOIRd.java
 *
 *
 *  @author Jean-Patrick Gelas
 */

import java.util.Properties;

public class TAMANOIRd
{
    static final int VERSION     = 0;
    static final int SUB_VERSION = 3;
    static boolean DBG = false ; // verbose mode if true

    /* ####################################### */
    /* For more information about this boolean */
    /* read README.JAVA file                   */
    /* ####################################### */
    static final boolean RUN_IN_JVM = true;
    /* ####################################### */

    static int port = 5432; // default port

    // address of the node hosting a web server (apache)
    static final String SERVICE_BROKER_ADDRESS = "140.77.15.124";

    // name of the frontend for a tamanoir cluster
    static String frontendName = null;
    static String interfaceName = null;

    // file containing the global topology of the AN Network
    static String table_dat = "../route/table.dat";
    static Route  route;

    // manage and list services on the local AN
    static ServiceManager serviceManager = new ServiceManager();
    static String serviceBroker = "http://" + SERVICE_BROKER_ADDRESS;
    static String service_path = "./";
    static String shared_obj_ext; // ".class" or ".so"

    // variables for statistics use
    static long recv_udp_counter       = 0; // number of received packets
    static long send_udp_counter       = 0; // number of sent packets
    static long outputsize_udp_counter = 0; // data size tranmitted
    static long inputsize_udp_counter  = 0; // data size received

    static long recv_tcp_counter       = 0; // number of received packets
    static long send_tcp_counter       = 0; // number of send packets
    static long outputsize_tcp_counter = 0; // data size tranmitted
    static long inputsize_tcp_counter  = 0; // data size received

    // max waiting request
    static int  queueSize = 100;



    public TAMANOIRd () 
    {
	display_intro();

	if(RUN_IN_JVM) 
	    shared_obj_ext = ".class"; /* when TAMANOIR run in a JVM */
        else 
	    shared_obj_ext = ".so";    /* when TAMANOIR run in native */

	route = new Route(TANet.getlocalInetAddress(), table_dat);
        //if(DBG) route.display();

        // run TCP server
        TANd_tcp s_tcp = new TANd_tcp( port );
        s_tcp.start();

	// another TCP server (able to receive raw data (not ANEP) (ex: http))
        TANd_raw s_raw = new TANd_raw( port + 1);
        s_raw.start();
	
	// run UDP server
        new TANd_udp( port ); // TANd_udp s_udp = new TANd_udp( port );
    }

    static void display_intro ()
    {
	System.out.println();
	System.out.println( "*-------------------*");
	System.out.println( "| TAMANOIRd - v " + 
			    VERSION + "." + SUB_VERSION + " |");
	System.out.println( "*-------------------*");
	Properties props = System.getProperties();
	System.out.print("Java version = "+props.getProperty("java.version"));
	if(RUN_IN_JVM) System.out.println( " [running in a JVM]" );
	else System.out.println( " [running in native]" );

	System.out.println("System = "+props.getProperty("os.name") + " " +
			   props.getProperty("os.version") + "/" +
			   props.getProperty("os.arch")
			   );
	System.out.println();
    }

    static void usage ()
    {
	System.out.println("Usage : TAMANOIRd [-sb name] [-p port] [-r table.dat] [--help] -fe name");
	System.out.println();
	System.out.println("-sb ip : service broker hostname");
	System.out.println("-fe ip : front-end name (used with LVS)");
	System.out.println("-if ip : for multi IP system (used with loopback addresses" + 
			   " for local test)");
    }

    static void parseArgs ( String[]args )
    {
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-p"))
		    {
			i++;
			port = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-r"))
		    {
			i++;
			table_dat = args[i];
		    }
 		else if (args[i].equals ("-sb"))
		    {
			i++;
			serviceBroker = "http://" + args[i];
		    }
		else if (args[i].equals ("-fe"))
		    {
			i++;
			frontendName = args[i];
			
		    }
		else if (args[i].equals ("-if"))
		    {
			i++;
			interfaceName = args[i];
			
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()



    public static void main (String[]args)
    {
	parseArgs (args);
	
	new TAMANOIRd();
    } 
} // class TAMANOIRd
