/**
 *  FptpManager.java : Fptp protocol deployment tool on Tamanoir EE.
 *
 *  @author Jean-Patrick Gelas, Ernesto Exposito
 *  @email  jpgelas@ens-lyon.fr, ernesto@ensica.fr
 *  @version 0.1
 */

import java.io.*;
import java.net.*;
import java.awt.*;

public class FptpManager 
{
    static final String VERSION = "0.1";

    static final boolean DBG = true;

    static String serviceName = "FptpS"; //default service name

    static String serviceBroker = "http://140.77.15.124"; 

    static String psTAN = null; // proxy server
    static String pcTAN = null; // proxy client
    static int portTAN  = 5432; // default port

    static int portServiceFptp = 20000; // default port is 20000

    static int proto = TANet.TCP; // default protocol is TCP

    static byte [] srvNameinBytes;

    static byte [] payload ;
    final static int INT_SIZE = 4; // (Bytes) 32 bits signed integer

    static TANet psnet, pcnet;

    FptpManager()
    {
    } 

    static void usage ()
    {
	System.out.println( "FptpManager - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : FptpManager [-h] -ps IP -pc IP"+
			    "[-port i]");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -ps IP   : proxy server");
	System.out.println( "  -pc IP   : proxy client");
	System.out.println( "  -port    : port opened by FptpS (default is 20000)");
	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			portServiceFptp = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-ps"))
		    {		
			i++;
			psTAN = args[i];
		    }
		else if (args[i].equals ("-pc"))
		    {		
			i++;
			pcTAN = args[i];
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name   : " + serviceName );
	System.out.println ("proxy Server   : " + psTAN );
	System.out.println ("proxy client   : " + pcTAN );
	System.out.println ("port Fptp service : " + portTAN );
        System.out.print   ("Protocole      : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
    }




    static public int sendFptpDeploymentPacket()
    {
	// build service name
	String sbsn = new String(serviceBroker + "/" + serviceName);
	srvNameinBytes    = sbsn.getBytes();
	
	try {
	    psnet.makeANEPkt( srvNameinBytes, // service name
			      psnet.srcId.getBytes(), //src
			      psTAN.getBytes(), // dst
			      psTAN.getBytes(), // last
			      psTAN.getBytes(), // proxy
			      payload );
	    psnet.sendANEPkt();	    

	    pcnet.makeANEPkt( srvNameinBytes, 
			      pcnet.srcId.getBytes(), 
			      pcTAN.getBytes(), 
			      pcTAN.getBytes(), 
			      pcTAN.getBytes(), 
			      payload );
	    pcnet.sendANEPkt();	    
	    

	} catch (IOException e) {
	    System.err.println( "FptpManager : " + e );
	}
	return 0;
    } // sendFptpDeploymentPacket()


    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(psTAN == null && pcTAN == null) {
	    usage ();
	    System.exit (1);
	}

	// Write the port used by service Fptp in a Byte Array
	payload = new byte[1*INT_SIZE];
	try {
	    ByteArrayOutputStream baos = 
		new ByteArrayOutputStream(1*INT_SIZE);
	    DataOutputStream dos = new DataOutputStream( baos );
	    dos.writeInt( portServiceFptp ); // port de FptpS
	    payload = baos.toByteArray();
	} catch (IOException e) {
	    System.err.println("FptpManager : " + e);
	}

	if(DBG) displayOptions();

	// build links with proxies
	psnet = new TANet( TANet.TCP, psTAN, portTAN );
	pcnet = new TANet( TANet.TCP, pcTAN, portTAN );

	// send command packet to force Fptp service deployment
	sendFptpDeploymentPacket();

	// and close links between the Fptp manager and proxies.
	psnet.close();
	pcnet.close();

    } // main()

	

}

