/**
 * RecvPackets.java
 *
 * @author Jean-Patrick Gelas
 *
 *
 */


/*

WARNING use this class with a transmitter embedding time stamp in
the beginning of the payload (see SendPackets.class, (long System.currentTimeMillis))

*/

import java.io.*;
import java.net.*;

public class RecvPackets extends Thread
{
    static final String VERSION = "0.1";

    static final boolean DBG = false;
    
    static int modulo = 100; // display info every MODULO sent packets
    static boolean quiet = false ;

    static final int MAX_SIZE = 1024 * 64;
    static byte[] payload = new byte[MAX_SIZE];

    static int proto = TANet.TCP; // default protocol is TCP

    static String serverName   = null;
    static int    serverPort   = 5432; // default port
    static String proxyTANname = null;
    static int    proxyTANport = 5432; // default port

    static String interfaceName = null;
    static int queueSize = 100; // ServerSocket

    static int numberOfStreams = 1;

    // UDP variables
    static DatagramSocket ds;

    RecvPackets()
    {
	this.start();
	if(DBG) displayOptions();
    }


    static void usage ()
    {
	System.err.println( "RecvPackets - v " + VERSION );
	System.err.println( "------------------------");
	System.err.println( "Usage : RecvPackets [-h] [-udp | -tcp] ");
	System.err.println( "        -tan proxyTAN [-tp p] -s serverName [-sp p]");
	System.err.println( "");
	System.err.println( "");
	System.err.println( "  -h       : this help");
	System.err.println( "  -tan s   : this host must be connected to a proxyTAN");
	System.err.println( "  -tp i    : proxyTAN port ["+proxyTANport+"]");
	System.err.println( "  -s s     : name of stream server");
	System.err.println( "  -sp i    : port used by stream server");
	

	System.err.println( "  -udp     : receive data on a UDP channel");
	System.err.println( "  -tcp     : receive data on a TCP channel (default)");
	System.err.println( "  -q       : quiet mode (no output)");
	System.err.println( "  -m i     : display info every i pkts received");
	System.out.println( "  -if IP   : local adress");
	System.out.println( "  -ns n    : number of streams (default 1)(TCP only)");
	System.err.println();
	System.err.println();
	System.err.println( "ex: ..." );
	System.err.println();
    }
  
    static void parseArgs ( String[]args )
    {
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-tp"))
                    {
                        i++;
                        proxyTANport = Integer.parseInt(args[i]);
                    }
                else if (args[i].equals ("-tan"))
                    {
                        i++;
                        proxyTANname = args[i];
                    }
                else if (args[i].equals ("-s"))
                    {
                        i++;
                        serverName = args[i];
                    }
		else if (args[i].equals ("-sp"))
                    {
                        i++;
                        serverPort = Integer.parseInt(args[i]);
                    }
		else if (args[i].equals ("-m"))
		    {		
			i++;
			modulo = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-q"))
		    {		
			quiet = true;
		    }
		else if (args[i].equals ("-if")) // use to fix address
                    {           
                        i++;
			interfaceName = args[i]; 
                    }
		else if (args[i].equals ("-ns"))
                    {
                        i++;
                        numberOfStreams = Integer.parseInt(args[i]);
                    }
		else
		    {
			System.err.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.err.println ("Server       : " + serverName );
	System.err.println ("Server port  : " + serverPort );
	System.err.println ("proxy TAN    : " + proxyTANname );
	System.err.println ("port  TAN    : " + proxyTANport );
	System.err.println ("quiet mode   : " + quiet );	
        System.err.print   ("protocole    : " );
	switch(proto) {
	case TANet.UDP : System.err.println("UDP"); break;
	case TANet.TCP : System.err.println("TCP"); break;
	default : System.err.println( "unknown" );
	}
    }


    // recvStream() : receive payload stream and write it on stdout
    public static int recvStream( DataInputStream rd, int streamNumber ) 
    {
	int cpt = 0;
	int c,size;// used by case TCP
	int count = 0, total_count = 0; 
	BufferedOutputStream stdout = null;
	if(!quiet) stdout = new BufferedOutputStream( System.out );

	// for time stamp read in payload
	long start_time = 0, total_time = 0;
	ByteArrayInputStream bais = new ByteArrayInputStream(payload);
	DataInputStream dis = new DataInputStream( bais );	


	try {
	    for(;;) {
		switch(proto) {
		case TANet.TCP:
		    size = rd.readInt(); // read size of data incoming
		    if(DBG) System.err.println( "RecvPacket size="+size);
		    count = 0;
		    while (count < size ) {
			c = rd.read( payload, count, size - count );
			if(c<0) break; // client socket has been closed
			count += c;
			// if(DBG) System.err.println( "recvStream (tcp):
			// transfer in progress = ["+count+"] + " + c );
		    }
		    if(DBG) System.err.println( "recvStream (tcp): reception"+
						" completed [" + count +
						" Bytes]" );
		    break;

		case TANet.UDP:
		    DatagramPacket dp = new DatagramPacket (payload, MAX_SIZE);
		    ds.receive (dp);
		    count = dp.getLength();
		    if(DBG) System.err.println( "recvStream (udp): reception"+
                                                " completed [" + count +
                                                " Bytes]" );

		    break;
		} //switch()
		total_count += count ;

		// and write its payload on stdout
		if(!quiet) {
		    stdout.write( payload, 0, count );
		    stdout.flush();
		}

		
		// get time stamp at the beginning of the payload
		if(start_time == 0) {
		    start_time = dis.readLong();
		} else {
		    if( cpt++ % modulo == 0 ) {
			dis.reset();
			long t = dis.readLong() - start_time;
			if(t>0) {
				System.err.print("[# "+ streamNumber + "] "+((8.0*total_count)/(t*1000)) + " Mbit/s,  ");
				System.err.println((((total_count / 4096)*1000)/t) + " pps (if 4096 Bytes/packet)");

			}
		    }
		}

		if(DBG) if( cpt % modulo == 0 )
		    System.err.println( "[# "+ streamNumber + "] RecvPackets : #" + cpt );

	    } // for(;;)
	} catch (IOException e) {
	    System.err.println("RecvPackets : " + e);
	}	
	return 1;
    }

    public void run()
    {
	try {

	    InetAddress  ia = null;
	    switch(proto) {
	    case TANet.TCP:
		ServerSocket ss;
		if(interfaceName == null)
		    ss = new ServerSocket( proxyTANport, queueSize );
		else {
		    ia = InetAddress.getByName(interfaceName);
		    ss = new ServerSocket( proxyTANport, queueSize, ia );
		}
		    
		System.err.println("RecvPackets (TCP): waiting for data on port " + proxyTANport);
		try {
		    for(int i=0; i < numberOfStreams; i++) {
			TANet.init_session(proxyTANname, proxyTANport, serverName, serverPort, ia);
			Socket s = ss.accept();
			ThreadClient t = new ThreadClient(s, i);
			t.start();
		    }
		}catch (IOException e) {
		    e.printStackTrace();
		} finally {
		    try {
			ss.close();
		    } catch (IOException e) {}
		}
		break;
		
	    case TANet.UDP:
		if(interfaceName == null)
		    ds = new DatagramSocket( proxyTANport ); 
		else {
		    ia = InetAddress.getByName(interfaceName);
		    ds = new DatagramSocket( proxyTANport, ia );
		}
		System.err.println("RecvPackets (UDP): waiting for data on port " + proxyTANport);
		TANet.init_session(proxyTANname, proxyTANport, serverName, serverPort, ia);
		if(ds != null) {
		    recvStream( null, 0 );
		} else
		    System.err.println("RecvPackets (udp): problem !");
		break;
	    } // switch()
	    System.err.println("RecvPackets : Thread termine.");
        } catch (IOException e) {
            System.err.println( "RecvPackets (run): " + e );
        }
    } // run()



    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(serverName == null || proxyTANname == null) {
	    usage (); 
	    System.exit (1);
	}
	new RecvPackets();
    } // main()
}

//Saad

class ThreadClient extends Thread {
    
    Socket clientSocket;   
    int streamNumber;
    
    public ThreadClient(Socket clientSocket, int streamNumber) throws IOException {
	this.clientSocket = clientSocket;
	this.streamNumber = streamNumber;
    }
    
    public void run() {  
	try {
	    DataInputStream rd=new DataInputStream(clientSocket.getInputStream());
	    RecvPackets.recvStream( rd, streamNumber );
	    
	} catch(IOException e) {
	    e.printStackTrace();
	} finally {
	    try {
		clientSocket.close();
	    } catch(Exception e) {
	    }
	}
    }
}






	// si pb dans la suite, il faudra liberer les deux sockets
	//	out=new PrintWriter(clientSocket.getOutputStream(),true);
	//	in=new BufferedReader(
	//	   new InputStreamReader(clientSocket.getInputStream()));



