/**
 *  TAMonitor.java : monitor of TAManoir node
 *
 *  @author Jean-Patrick Gelas
 *  @email jpgelas@ens-lyon.fr
 *  @version 0.1
 */

import java.io.*;
import java.net.*;
import java.awt.*;

public class TAMonitor // extends Frame
{
    static final String VERSION = "0.1";

    static final boolean DBG = true;

    static String serviceName = "MonitorS"; //default service

    static String serviceBroker = "http://140.77.15.124"; 
    static int delay = 2000; // default delay (ms) between each sent packet


    static boolean infiniteLoop = true;

    static String proxyTAN = null;
    static int proxyTANport = 5432; // default port
    static int serverPort = 5432;   // default port

    static int proto = TANet.TCP; // default protocol is TCP
    static String destId = null;

    static boolean withGUI = false;
    static Frame   frame;
    static Container cont;
    static Dimension d;
    static TAMonitorGUI monit;

    static byte [] srvNameinBytes;
    static byte [] srcIdinBytes  ;   
    static byte [] destIdinBytes ;   
    static byte [] lastIdinBytes ;   
    static byte [] proxyTANIdinBytes;

    static byte [] payload ;
    final static int INT_SIZE = 4; // (Bytes) 32 bits signed integer

    static TANet net;

    // variables for statistics use
    static long recv_udp_counter = 0;       // number of received packets
    static long send_udp_counter = 0;       // number of sent packets
    static long outputsize_udp_counter = 0; // data size tranmitted
    static long inputsize_udp_counter  = 0; // data size received

    static long recv_tcp_counter = 0;       // number of received packets
    static long send_tcp_counter = 0;       // number of send packets
    static long outputsize_tcp_counter = 0; // data size tranmitted
    static long inputsize_tcp_counter  = 0; // data size received

    TAMonitor()
    {
	if(withGUI) {
	    cont = new Container();	
	    monit = new TAMonitorGUI();
	    d = monit.getPreferredSize();
	    monit.setBackground(Color.black);
	    
	    frame = new Frame();
	    frame.setSize(d);
	    frame.add(monit);
	}
    } 

    static void usage ()
    {
	System.out.println( "TAMonitor - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : TAMonitor [-h] [-delay d] "+
			    "[-port p] [-gui] -tan proxyTAN -t target");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to "+
			    "a proxyTAN");
	System.out.println( "  -t target: monitored TAN");
	System.out.println( "  -delay d : refresh info frequency (ms)");
	System.out.println( "  -port p  : send data on port p of proxyTAN");
	System.out.println( "  -gui     : graphical interface");

	System.out.println();
    } // usage()
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-t"))
		    {		
			i++;
			destId = args[i];
		    }
		else if (args[i].equals ("-gui"))
		    {		
			withGUI = true;
		    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name   : " + serviceName );
	System.out.println ("Service Broker : " + serviceBroker );
	System.out.println ("Delay          : " + delay );
	System.out.println ("proxy TAN      : " + proxyTAN );
	System.out.println ("port  TAN      : " + proxyTANport );
	System.out.println ("target         : " + destId);
        System.out.print   ("Protocole      : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
    }




    static public int sendDelayPackets()
    {

	long t0 = 0;
	long old_inputsize_udp_counter = 0;
	long old_outputsize_udp_counter = 0;
	long old_inputsize_tcp_counter = 0; 
	long old_outputsize_tcp_counter = 0; 
	double UDP_recv, UDP_send, TCP_recv, TCP_send;
	// cache and init some data
	String sbsn = new String(serviceBroker + "/" + serviceName);
	srvNameinBytes    = sbsn.getBytes();
	srcIdinBytes      = net.srcId.getBytes();
	destIdinBytes     = destId.getBytes(); 
	lastIdinBytes     = srcIdinBytes;
	proxyTANIdinBytes = proxyTAN.getBytes(); 
	
	try {
	    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
			    destIdinBytes, lastIdinBytes, 
			    proxyTANIdinBytes, payload );
	    net.sendANEPkt();	    

	    while( true ) {
		net.recvANEPkt();
		net.recoverInfoFromANEPkt();
		ByteArrayInputStream bais = 
		    new ByteArrayInputStream( net.getANEPkt().payload );
		DataInputStream dis = new DataInputStream( bais );
		System.out.println("Info from : [ " + net.srcId + 
				   " ] ------");
		
		recv_udp_counter       = dis.readLong(); 
   		inputsize_udp_counter  = dis.readLong(); 
		send_udp_counter       = dis.readLong(); 
		outputsize_udp_counter = dis.readLong();

		recv_tcp_counter       = dis.readLong(); 
		inputsize_tcp_counter  = dis.readLong();
		send_tcp_counter       = dis.readLong();
		outputsize_tcp_counter = dis.readLong();

		long t_now = System.currentTimeMillis();
		long t = t_now - t0;

		if( t0 != 0 && t != 0) {
		    long t_udprecv = (((inputsize_udp_counter-old_inputsize_udp_counter)*8)/t);
		    long t_udpsend = (((outputsize_udp_counter-old_outputsize_udp_counter)*8)/t);
		    long t_tcprecv = (((inputsize_tcp_counter-old_inputsize_tcp_counter)*8)/t);
		    long t_tcpsend = (((outputsize_tcp_counter-old_outputsize_tcp_counter)*8)/t);

		    System.out.println("UDP :recv " + recv_udp_counter + 
				       " ["+ inputsize_udp_counter +"]" + " " +  
				       t_udprecv + " Kbit/s");
		    
		    System.out.println("     send " + send_udp_counter + 
				       " ["+ outputsize_udp_counter +"]" + " " + 
				       t_udpsend + " Kbit/s");
		    
		    System.out.println("TCP :recv " + recv_tcp_counter  + 
				       " ["+ inputsize_tcp_counter +"]" + " " + 
				       t_tcprecv + " Kbit/s");
		    
		    System.out.println("     send " + send_tcp_counter  + 
				       " ["+ outputsize_tcp_counter +"]" + " " + 
				       t_tcpsend + " Kbit/s");

		    if(monit != null) {
			monit.addPoints(t_udpsend, t_udprecv, t_tcpsend, t_tcprecv);
			monit.repaint();
		    }
		}
		t0 = t_now;
		old_inputsize_udp_counter = inputsize_udp_counter;
		old_outputsize_udp_counter = outputsize_udp_counter;
		old_inputsize_tcp_counter = inputsize_tcp_counter;
		old_outputsize_tcp_counter = outputsize_tcp_counter;
	    } // while( true )
	} catch (BadANEPversionException e) {
	    System.err.println( "TAMonitor : recvANEPkt() : Received " +
				"unsupported ANEP version exception : "
				+e);
	} catch (IOException e) {
	    System.err.println( "TAMonitor : recvANEPkt() : " + e );
	} catch (InterruptedException e) {
	    System.err.println("TAMonitor : " + e );
	}
	return 0;
    } // sendDelayPackets()


    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null || destId == null) {
	    usage ();
	    System.exit (1);
	}


	payload = new byte[1*INT_SIZE];
	try {
	    ByteArrayOutputStream baos = 
		new ByteArrayOutputStream(1*INT_SIZE);
	    DataOutputStream dos = new DataOutputStream( baos );
	    dos.writeInt( delay );
	    payload = baos.toByteArray();
	    if(DBG) displayOptions();
	} catch (IOException e) {
	    System.err.println("TAMonitor : " + e);
	}

	new TAMonitor();
	
	if(withGUI) frame.show();
	if(monit != null) monit.setVisible(true);
	net = new TANet( TANet.TCP, proxyTAN, proxyTANport );

	// and finally send a delay packets
	sendDelayPackets();
	net.close();

    } // main()

	

}

