/* draws the curves for the TAMANOIR monitoring tool*/


import java.awt.*;
import java.lang.*;


public class TAMonitorGUI extends Canvas {

    static final boolean DBG = true ;
    
    static final int UDPSEND = 0;
    static final int UDPRECV = 1;
    static final int TCPSEND = 2;
    static final int TCPRECV = 3;

    public static int arraySize = 51; /*number of points*/
    /*pointers on the array*/
    public static int first;
    public static int last;
    public static int next;

    static double[][] ptArray = new double[arraySize][4];

    /*corresponding of the real size and the max that is displayed*/
    public static int scale;

    /*real size*/
    static int x_size = 400;
    static int y_size = 200;

    
    public Dimension getMinimumSize() {
	return new Dimension(450, 250); 
    }
    public Dimension getPreferredSize() {
	return new Dimension(450, 250); 
    }
    public Dimension getMaximumSize() {
	return new Dimension(450, 250); 
    }

    /* adds the new points in the array of points */
    public void addPoints(double udpSend, 
			  double udpRecv,
			  double tcpSend,
			  double tcpRecv)
    {
	
	/*if this is the first call */
	if (last == next)
	    next = (next + 1) % arraySize;
	else {
	    last = next;
	    next = (next + 1) % arraySize;	    
	    /*if the array is already full*/
	    if (first == last)
		first = (first + 1) % arraySize;
	}	    
	
	System.out.println("first : " + first);
	ptArray[last][UDPSEND] = udpSend;
	ptArray[last][UDPRECV] = udpRecv;
	ptArray[last][TCPSEND] = tcpSend;
	ptArray[last][TCPRECV] = tcpRecv;

	scale = setScale();	
    }//addPoints


    /*sets the scale to 10, 50, 100, 500 ,... */
    public int setScale()
    {
	int i, j;
	double max = 0;
	/* max of the array*/
	for (i=0;i < arraySize;i++)
	    for (j=0;j<4;j++){
		if (ptArray[i][j]>max)
		    max = ptArray[i][j];
	    }
       	int step = 5;
	int scal = 10;
	while (scal < max){
	    scal = scal * step;
	    if (step == 2)
		step = 5;
	    else
		step = 2;
	}
	return scal;
    }//setScale
    


    /* draws 1 of the lines in the given color */
    public void drawLn(Graphics g, int numb, Color c)
    {
	int i = first;
	double currPt = ptArray[i][numb];
	double nextPt;

	int Cx, Cy, Nx, Ny; /*location of the points*/
	int count = 0;
	
	double x_step;

	/* the line is drawn only if the array is not empty */
	if (first != last){

	    g.setColor(c);
	    
	    while (i!=last){
		i=(i+1) % arraySize;
		nextPt = ptArray[i][numb];
		
		x_step = x_size / ( arraySize - 1 );
		
		Cx = (int)(count * x_step + 50);
		Nx = (int)(Cx + x_step);
		Cy = (int) (y_size - (( currPt / scale ) * y_size ));
		Ny = (int) (y_size - (( nextPt / scale ) * y_size ));
		
		g.drawLine(Cx,Cy,Nx,Ny);
		
		currPt = nextPt;
		count ++;
	    }//while
	}//if

	Double t = new Double(currPt);
	int x_pos = 50 + 200 * (numb / 2);
	int y_pos = 220 + 20 * (numb % 2);
	String to_write = null;
	switch(numb) {
	case UDPSEND :
	    to_write = "UDP Send : " + t.toString();
	    break;
	case UDPRECV:
	    to_write = "UDP Recv : " + t.toString();
	    break;
	case TCPSEND:
	    to_write = "TCP Send : " + t.toString();
	    break;
	case TCPRECV:
	    to_write = "TCP Recv : " + t.toString();
	}
	g.drawString(to_write  + " kbit/s",x_pos,y_pos);
	
    }//drawLn
    

    /* draws the border */
    public void drawBd(int scal, Graphics g)
    {
	g.setColor(Color.white);
	g.drawLine(50, 0, 50, y_size);
	g.drawLine(50, y_size, x_size+50, y_size);

	// Y-axis legend 
	g.drawString(new Integer(scal).toString(),0,10);
	g.drawString("kbit/s",0,20);
    }


    /*main function*/
    public void paint(Graphics g) {
	Dimension d = this.getSize();
	drawBd(scale,g);
	drawLn(g, UDPSEND, Color.magenta);
	drawLn(g, UDPRECV, Color.red);
	drawLn(g, TCPSEND, Color.blue);
	drawLn(g, TCPRECV, Color.yellow);
    }

}
